Package: hmrc
Title: Download and Tidy 'HMRC' Statistical Data
Version: 0.4.0
Authors@R: 
    person("Charles", "Coverdale", , "charlesfcoverdale@gmail.com",
           role = c("aut", "cre"))
Description: Provides functions to download, parse, and tidy statistical
    data published by 'HM Revenue and Customs' ('HMRC') on 'GOV.UK'.
    Returns annotated 'hmrc_tbl' data frames with provenance metadata
    (source URL, fetch time, vintage, cell methods) for reproducible
    fiscal research. Covers monthly tax receipts (41 tax heads from 2008),
    'VAT' (from 1973), fuel duties (from 1990), tobacco duties (from 1991),
    annual 'Corporation Tax' receipts, stamp duty, research and development
    tax credit statistics (from 2000), tax gap estimates, 'Income Tax'
    liabilities by income range, and monthly property transaction counts.
    File URLs are resolved at runtime via the 'GOV.UK' Content API
    <https://www.gov.uk/api/content>, so data is always current without
    hardcoded URLs. Files are cached locally between sessions.
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-GB
RoxygenNote: 7.3.3
Imports: cli (>= 3.6.0), httr2 (>= 1.0.0), lifecycle (>= 1.0.0),
        readODS (>= 2.3.0), readxl (>= 1.4.0), tools, utils
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), ggplot2, scales, withr
Config/testthat/edition: 3
VignetteBuilder: knitr
URL: https://charlescoverdale.github.io/hmrc/,
        https://github.com/charlescoverdale/hmrc
BugReports: https://github.com/charlescoverdale/hmrc/issues
Depends: R (>= 4.1.0)
LazyData: true
NeedsCompilation: no
Packaged: 2026-04-26 13:14:18 UTC; charlescoverdale
Author: Charles Coverdale [aut, cre]
Maintainer: Charles Coverdale <charlesfcoverdale@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-26 13:30:03 UTC
Built: R 4.6.0; ; 2026-04-27 00:15:18 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method("[",hmrc_tbl)
S3method(as.data.frame,hmrc_tbl)
S3method(format,hmrc_tbl)
S3method(print,hmrc_tbl)
export(clear_cache)
export(get_corporation_tax)
export(get_fuel_duties)
export(get_income_tax_stats)
export(get_property_transactions)
export(get_rd_credits)
export(get_stamp_duty)
export(get_tax_gap)
export(get_tax_receipts)
export(get_tobacco_duties)
export(get_vat)
export(hmrc_cache_info)
export(hmrc_capital_gains)
export(hmrc_clear_cache)
export(hmrc_corporation_tax)
export(hmrc_creative_industries)
export(hmrc_fuel_duties)
export(hmrc_income_tax_stats)
export(hmrc_inheritance_tax)
export(hmrc_list_tax_heads)
export(hmrc_meta)
export(hmrc_patent_box)
export(hmrc_property_transactions)
export(hmrc_publications)
export(hmrc_rd_credits)
export(hmrc_search)
export(hmrc_stamp_duty)
export(hmrc_tax_gap)
export(hmrc_tax_receipts)
export(hmrc_tobacco_duties)
export(hmrc_vat)
export(list_tax_heads)

# hmrc

[![CRAN status](https://www.r-pkg.org/badges/version/hmrc)](https://CRAN.R-project.org/package=hmrc) [![CRAN downloads](https://cranlogs.r-pkg.org/badges/hmrc)](https://cran.r-project.org/package=hmrc) [![Total Downloads](https://cranlogs.r-pkg.org/badges/grand-total/hmrc)](https://CRAN.R-project.org/package=hmrc) [![Lifecycle: stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable) [![License: MIT](https://img.shields.io/badge/License-MIT-blue.svg)](https://opensource.org/licenses/MIT)

An R package for accessing statistical data published by [HM Revenue and Customs](https://www.gov.uk/government/organisations/hm-revenue-customs).

## What is HMRC?

HM Revenue and Customs is the UK government department responsible for collecting taxes, paying certain forms of state support, and enforcing customs rules. It is the single largest gatherer of government revenue: in 2023-24, HMRC collected around GBP 830bn in taxes and duties, roughly 90% of all government receipts.

The distinction between HMRC and the OBR matters for anyone working with UK fiscal data. HM Treasury *sets* fiscal policy: it decides tax rates and spending plans. The OBR *forecasts* fiscal outcomes independently. HMRC *reports* what actually came in, the cash receipts against which those plans and forecasts are measured. If you want to know what the government intended to raise, use the OBR. If you want to know what it actually raised, use HMRC.

HMRC publishes monthly receipts data covering every major tax and duty (Income Tax, VAT, NICs, Corporation Tax, fuel duties, stamp duties, alcohol and tobacco duties, and more) and annual statistics on liabilities, reliefs, and the tax gap. This is some of the most closely watched economic data published by the UK government. It moves markets, informs fiscal policy debates, and is widely cited in journalism, think-tank analysis, and parliamentary briefings.

---

## Why does this package exist?

HMRC's statistical data is freely available at [gov.uk](https://www.gov.uk/government/organisations/hm-revenue-customs/about/statistics). The problem is how it is available.

Every file is an ODS spreadsheet. Every file's download URL contains a random media hash that changes with each publication cycle, meaning hardcoded URLs stop working every month. There is no API. Getting the data into R requires knowing the right URL pattern, navigating the GOV.UK publication pages manually, reading an ODS file with non-standard headers, pivoting wide-format sheets into long format, and standardising column names. You do this every month.

This package does all of that automatically. Download URLs are resolved at runtime via the GOV.UK Content API, so data is always current. One function call returns a clean, tidy data frame. Data is cached locally so subsequent calls are instant. Every result is returned as an `hmrc_tbl` carrying provenance metadata (source URL, fetch time, vintage, cell methods) for reproducible fiscal research.

```r
library(hmrc)
hmrc_tax_receipts()
```

---

## Installation

```r
install.packages("hmrc")

# Or install the development version from GitHub
# install.packages("devtools")
devtools::install_github("charlescoverdale/hmrc")
```

---

## Functions

### Data fetchers

| Function | Description | Time series |
|---|---|---|
| `hmrc_tax_receipts()` | Monthly cash receipts for 41 tax heads (Income Tax, NICs, VAT, CT, duties, etc.) | Apr 2008 onwards |
| `hmrc_vat()` | Monthly VAT receipts (payments, repayments, import VAT, home VAT) | Apr 1973 onwards |
| `hmrc_fuel_duties()` | Monthly hydrocarbon oil duty receipts (petrol, diesel, other) | Jan 1990 onwards |
| `hmrc_tobacco_duties()` | Monthly tobacco duty receipts (cigarettes, cigars, hand-rolling, other) | Jan 1991 onwards |
| `hmrc_corporation_tax()` | Annual CT receipts by levy (onshore, offshore, Bank Levy, RPDT, EPL, EGL) | 2019-20 onwards |
| `hmrc_stamp_duty()` | Annual stamp duty receipts (SDLT, SDRT, stamp duty on documents) | 2003-04 onwards |
| `hmrc_rd_credits()` | Annual R&D tax credit claims and cost (SME and RDEC schemes) | 2000-01 onwards |
| `hmrc_tax_gap()` | Cross-sectional tax gap estimates by tax type, taxpayer group, behaviour | Most recent year |
| `hmrc_income_tax_stats()` | Annual Income Tax liabilities by income range (Table 2.5) | 2022-23 onwards |
| `hmrc_property_transactions()` | Monthly residential and non-residential transactions by UK nation | Apr 2005 onwards |
| `hmrc_capital_gains()` | Annual CGT taxpayers, gains, and tax liabilities (Table 1) | 1987-88 onwards |
| `hmrc_inheritance_tax()` | IHT estates, tax due, average tax, and effective rate by net-estate band | Latest year of death |
| `hmrc_patent_box()` | Annual companies electing into the Patent Box and total relief | 2013-14 onwards |
| `hmrc_creative_industries()` | Annual reliefs across eight creative-industries sectors | Sector-dependent |

### Discovery and infrastructure

| Function | Description |
|---|---|
| `hmrc_search()` | Keyword search of the dataset catalogue |
| `hmrc_publications()` | Index of implemented and planned publications |
| `hmrc_list_tax_heads()` | Lookup table of 41 tax-receipts identifiers (no download required) |
| `hmrc_meta()` | Extract provenance metadata from any `hmrc_tbl` result |
| `hmrc_cache_info()` | Inspect locally cached files |
| `hmrc_clear_cache()` | Delete locally cached files |

The pre-0.4.0 `get_*` names continue to work as deprecated aliases; they emit a one-time-per-session warning and will be removed in v0.6.0.

---

## Examples

### `hmrc_tax_receipts()` — monthly tax head receipts

```r
library(hmrc)

# Most recent month's receipts, ranked by size
receipts <- hmrc_tax_receipts()
latest   <- receipts[receipts$date == max(receipts$date), c("tax_head", "receipts_gbp_m")]
latest   <- latest[order(-latest$receipts_gbp_m), ]
head(latest, 6)
#>           tax_head receipts_gbp_m
#>     total_receipts          79432
#>         income_tax          24819
#>         nics_total          14237
#>                vat          13461
#>    corporation_tax           9147
#>          fuel_duty           2094
```

---

### `hmrc_meta()` — provenance metadata

Every fetcher returns an `hmrc_tbl` carrying source URL, fetch time, vintage, cell methods, and frequency:

```r
receipts <- hmrc_tax_receipts(tax = "vat", start = "2024-01")
hmrc_meta(receipts)
#> $dataset
#> [1] "tax_receipts_monthly"
#> $source_url
#> [1] "https://www.gov.uk/government/statistics/hmrc-tax-and-nics-receipts-for-the-uk"
#> $cell_methods
#> [1] "cash"
#> $frequency
#> [1] "monthly"
#> $fetched_at
#> [1] "2026-04-26 09:00:00 UTC"
```

`as.data.frame()` strips the metadata for downstream tidyverse use; subsetting with `[` preserves it.

---

### `hmrc_search()` — discover datasets

```r
# Anything in the catalogue mentioning capital gains
hmrc_search("capital gains")

# Only annual datasets already implemented
hmrc_search(implemented = TRUE, frequency = "annual")

# Roadmap items not yet exposed by an hmrc_* function
hmrc_search(implemented = FALSE)
```

---

### `hmrc_list_tax_heads()` — available tax head identifiers

```r
# See all 41 series available in hmrc_tax_receipts()
hmrc_list_tax_heads()
#>               tax_head                                    description   category available_from
#>         total_receipts                            Total HMRC receipts      total           2016
#>             income_tax         Income Tax (PAYE and Self Assessment)     income           2016
#>      capital_gains_tax                              Capital Gains Tax     income           2016
#>        inheritance_tax                              Inheritance Tax      income           2016
#>     apprenticeship_levy                           Apprenticeship Levy     income           2017
#>             nics_total  National Insurance Contributions (all classes)   nics           2016
#>  ...
```

---

### `hmrc_vat()` — monthly VAT receipts

```r
# VAT receipts vs repayments since 2020
vat <- hmrc_vat(measure = c("total", "repayments"), start = "2020-01")

# Monthly net VAT: repayments reduce the total
head(vat[vat$measure == "repayments", c("date", "receipts_gbp_m")], 4)
#>         date receipts_gbp_m
#>   2020-01-01          -9823   # repayments are negative
#>   2020-02-01          -8941
#>   2020-03-01          -9107
#>   2020-04-01          -7234   # repayments fell during lockdown
```

---

### `hmrc_fuel_duties()` — monthly hydrocarbon oil duty

```r
# Total fuel duty since 2010, a slow structural decline
fuel <- hmrc_fuel_duties(fuel = "total", start = "2010-01")

# Aggregate to annual
fuel$year <- format(fuel$date, "%Y")
annual <- aggregate(receipts_gbp_m ~ year, data = fuel, FUN = sum)
tail(annual, 6)
#>   year receipts_gbp_m
#>   2019          27832
#>   2020          22145   # COVID lockdowns, far less driving
#>   2021          24917
#>   2022          24601
#>   2023          23884
#>   2024          23012
```

---

### `hmrc_tobacco_duties()` — monthly tobacco duty by product

```r
tobacco <- hmrc_tobacco_duties(
  product = c("cigarettes", "hand_rolling"),
  start   = "2015-01"
)

# Annual totals: hand-rolling has grown as cigarettes decline
tobacco$year <- format(tobacco$date, "%Y")
agg <- aggregate(receipts_gbp_m ~ year + product, data = tobacco, FUN = sum)
agg[agg$year == "2024", ]
#>   year      product receipts_gbp_m
#>   2024  cigarettes           6941
#>   2024 hand_rolling          1298
```

---

### `hmrc_capital_gains()` — annual CGT taxpayers, gains, liabilities

```r
# Total CGT receipts in recent years
cgt <- hmrc_capital_gains(measure = "tax_total_gbp_m")
tail(cgt[, c("tax_year", "value")], 5)
#>   tax_year  value
#>    2019-20   9803
#>    2020-21  14282
#>    2021-22  16672
#>    2022-23  14391
#>    2023-24  13316
```

---

### `hmrc_inheritance_tax()` — IHT estates by net-estate band

```r
# Number of taxpaying estates by band, latest year of death
iht <- hmrc_inheritance_tax()
iht[iht$measure == "number_taxed" & iht$estate_band != "Total",
    c("estate_band", "value")]
#>      estate_band value
#>     GBP 0-100k        0
#>   GBP 100k-200k      32
#>   ...
#>      GBP 10m+        42
```

---

### `hmrc_patent_box()` — Patent Box elections and relief

```r
hmrc_patent_box()
#>   tax_year companies relief_gbp_m
#>    2013-14       710          365
#>    2014-15       925          376
#>    ...
#>    2022-23      1735         1469
```

---

### `hmrc_creative_industries()` — film, TV, games, theatre, etc.

```r
# Film tax relief over time
hmrc_creative_industries(sector = "film")

# All eight sectors in the latest year
hmrc_creative_industries(tax_year = "2023-24")
```

---

### `hmrc_stamp_duty()` — annual stamp duty receipts

```r
sd <- hmrc_stamp_duty()
sd[sd$tax_year %in% c("2019-20", "2020-21", "2021-22", "2022-23", "2023-24") &
   sd$type == "sdlt_total", c("tax_year", "receipts_gbp_m")]
#>   tax_year receipts_gbp_m
#>    2019-20          11689
#>    2020-21           8670   # SDLT holiday (less tax paid on property)
#>    2021-22          15312   # holiday tapering off, boom in transactions
#>    2022-23          15381
#>    2023-24          11628   # higher rates cooling the market
```

---

### `hmrc_corporation_tax()` — annual CT receipts by levy type

```r
ct <- hmrc_corporation_tax()
ct[ct$tax_year == "2024-25", c("type", "receipts_gbp_m")]
#>                          type receipts_gbp_m
#>           all_corporate_taxes          94765
#>                     bank_levy           1520
#>                bank_surcharge           2891
#>   electricity_generators_levy            340
#>           energy_profits_levy           2645
#>                   offshore_ct           3210
#>                    onshore_ct          81440
#>                          rpdt            415
#>                      total_ct          88095
```

---

### `hmrc_rd_credits()` — R&D tax credit claims and cost

```r
# Cost of R&D tax credits by scheme: SME vs RDEC
rd <- hmrc_rd_credits(measure = "amount_gbp_m")
rd[rd$tax_year %in% c("2019-20", "2020-21", "2021-22", "2022-23", "2023-24"), ]
#>   tax_year scheme description     measure value
#>    2019-20    sme  SME R&D Relief amount_gbp_m  4385
#>    2020-21    sme  SME R&D Relief amount_gbp_m  4690
#>    2021-22    sme  SME R&D Relief amount_gbp_m  4620
#>    2022-23    sme  SME R&D Relief amount_gbp_m  4440
#>    2023-24    sme  SME R&D Relief amount_gbp_m  3145   # reform impact
```

---

### `hmrc_tax_gap()` — tax gap estimates

```r
gap <- hmrc_tax_gap()

# Largest gaps by absolute value
gap_sorted <- gap[order(-gap$gap_gbp_bn), c("tax", "component", "gap_gbp_bn", "uncertainty")]
head(gap_sorted, 6)
```

---

### `hmrc_income_tax_stats()` — Income Tax liabilities by income range

```r
it <- hmrc_income_tax_stats(tax_year = "2023-24")
it[, c("income_range", "taxpayers_thousands", "tax_liability_gbp_m", "average_rate_pct")]
#>   income_range taxpayers_thousands tax_liability_gbp_m average_rate_pct
#>          12570                2960                 627              1.5
#>          15000                5490                4640              4.9
#>          20000               10200               22500              9.0
#>          30000               10600               50200             12.4
#>          50000                5800               71300             18.7
#>         100000                 922               32000             29.1
#>         150000                 315               18400             34.0
#>         200000                 312               33900             38.0
#>         500000                  54               14700             40.6
#>        1000000                  18                9840             40.7
#>       2000000+                   9               19400             39.6
#>     All Ranges               36600              277000             18.1
```

---

### `hmrc_property_transactions()` — monthly transaction counts

```r
sdlt <- hmrc_property_transactions(
  type   = "residential",
  nation = "england",
  start  = "2021-01",
  end    = "2022-06"
)
sdlt[sdlt$date %in% as.Date(c("2021-03-01", "2021-06-01", "2021-10-01")),
     c("date", "transactions")]
#>         date transactions
#>   2021-03-01       147390   # rush before first SDLT-holiday deadline
#>   2021-06-01       192510   # rush before extended deadline
#>   2021-10-01        78200   # holiday ends, volumes normalise
```

---

## Caching

All downloads are cached locally in your user cache directory. Subsequent calls return the cached copy instantly with no network request.

```r
# Force a fresh download by setting cache = FALSE
hmrc_tax_receipts(cache = FALSE)

# Inspect the local cache
hmrc_cache_info()

# Remove files older than 30 days
hmrc_clear_cache(max_age_days = 30)

# Remove all cached files
hmrc_clear_cache()
```

---

## How URL resolution works

HMRC data files are hosted on `assets.publishing.service.gov.uk` with a random media hash in the path that changes every publication cycle. This makes hardcoding URLs impossible.

This package queries the [GOV.UK Content API](https://content-api.publishing.service.gov.uk/) at runtime to discover the current download URL for each publication, then caches the file locally. This means:

- Data is always current: the day HMRC publishes a new monthly bulletin, the next call to a fetcher will download the updated file.
- No manual maintenance is needed to handle URL rotation.
- A network connection is required for the first call; subsequent calls use the cache.

---

## Limitations

- **Provisional vintages.** The latest one or two tax years in CGT, R&D, and Creative Industries series are flagged provisional by HMRC and are revised in subsequent publications as late returns and claims arrive. The `status` column on Creative Industries carries the HMRC revision label.
- **Suppressed cells.** HMRC suppresses cells where small sample sizes risk identifying taxpayers (`[c]`) or where the value is structurally absent (`[z]` for IHT estates below the nil-rate band). These return `NA`.
- **Publication lag.** Inheritance Tax statistics carry a roughly three-year administrative lag (latest is 2022-23 deaths, published 2025). This package returns the latest published vintage; older years are not exposed.
- **Slug churn.** A handful of HMRC publications change their landing-page slug on each release (e.g. `corporation-tax-statistics-2025`, `creative-industries-statistics-august-2025`). The package sweeps recent candidate slugs; if HMRC moves to a substantially different naming scheme the package will fail loudly until updated.
- **Network at first call.** Fetchers require an internet connection on first call to resolve the GOV.UK Content API and download the file. Subsequent calls in the same session use the cache.
- **Scope.** This package wraps published HMRC tabular statistics. It does not provide microdata access (see the [`taxstats`](https://github.com/HughParsonage/taxstats) package for SPI microdata) and does not implement microsimulation (see the UKMOD framework). There is no equivalent Python package on PyPI as of April 2026.

---

## Citation

```r
citation("hmrc")
```

A `CITATION.cff` file is also provided at the repo root for the GitHub citation widget and Zenodo deposits.

---

## Related packages

This package is part of a suite of R packages for economic, financial, and policy data. They share a consistent interface (named functions, tidy data frames, local caching, provenance metadata) and are designed to work together.

**Data access:**

| Package | Source |
|---|---|
| [`ons`](https://github.com/charlescoverdale/ons) | UK Office for National Statistics |
| [`boe`](https://github.com/charlescoverdale/boe) | Bank of England |
| [`obr`](https://github.com/charlescoverdale/obr) | Office for Budget Responsibility |
| [`ukhousing`](https://github.com/charlescoverdale/ukhousing) | UK Land Registry, EPC, Planning |
| [`fred`](https://github.com/charlescoverdale/fred) | US Federal Reserve (FRED) |
| [`readecb`](https://github.com/charlescoverdale/readecb) | European Central Bank |
| [`readoecd`](https://github.com/charlescoverdale/readoecd) | OECD |
| [`readnoaa`](https://github.com/charlescoverdale/readnoaa) | NOAA Climate Data |
| [`readaec`](https://github.com/charlescoverdale/readaec) | Australian Electoral Commission |
| [`comtrade`](https://github.com/charlescoverdale/comtrade) | UN Comtrade |
| [`carbondata`](https://github.com/charlescoverdale/carbondata) | Carbon markets (EU ETS, UK ETS, voluntary registries) |

**Analytical toolkits:**

| Package | Purpose |
|---|---|
| [`inflateR`](https://github.com/charlescoverdale/inflateR) | Inflation adjustment for price series |
| [`inflationkit`](https://github.com/charlescoverdale/inflationkit) | Inflation analysis (decomposition, persistence, Phillips curve) |
| [`yieldcurves`](https://github.com/charlescoverdale/yieldcurves) | Yield curve fitting (Nelson-Siegel, Svensson) |
| [`debtkit`](https://github.com/charlescoverdale/debtkit) | Debt sustainability analysis |
| [`nowcast`](https://github.com/charlescoverdale/nowcast) | Economic nowcasting |
| [`predictset`](https://github.com/charlescoverdale/predictset) | Conformal prediction |
| [`climatekit`](https://github.com/charlescoverdale/climatekit) | Climate indices |
| [`inequality`](https://github.com/charlescoverdale/inequality) | Inequality and poverty measurement |

---

## Issues

Please report bugs or requests at <https://github.com/charlescoverdale/hmrc/issues>.

---

## Keywords

HMRC, UK tax data, tax revenue, VAT, income tax, corporation tax, capital gains tax, inheritance tax, patent box, creative industries, R&D tax credits, stamp duty, alcohol duty, tobacco duty, fuel duty, R package, UK government data, fiscal data

#' Inspect the local HMRC cache
#'
#' Returns a tidy table of files currently held in the package cache, with
#' size and age. Useful for understanding what is stored locally and deciding
#' when to clear stale files via [hmrc_clear_cache()].
#'
#' @return A data frame with columns `file`, `extension`, `size_bytes`,
#'   `size_mb`, `modified` (POSIXct), `age_days`, `path`. The data frame is
#'   empty (zero rows) if the cache directory does not exist or is empty.
#'   The cache directory itself is attached as the attribute `"cache_dir"`.
#'
#' @examples
#' \donttest{
#' op <- options(hmrc.cache_dir = tempdir())
#' info <- hmrc_cache_info()
#' attr(info, "cache_dir")
#' options(op)
#' }
#'
#' @family infrastructure
#' @export
hmrc_cache_info <- function() {
  cache_dir <- hmrc_cache_dir()

  empty <- data.frame(
    file        = character(0),
    extension   = character(0),
    size_bytes  = numeric(0),
    size_mb     = numeric(0),
    modified    = as.POSIXct(character(0)),
    age_days    = numeric(0),
    path        = character(0),
    stringsAsFactors = FALSE
  )
  attr(empty, "cache_dir") <- cache_dir

  if (!dir.exists(cache_dir)) return(empty)

  files <- list.files(cache_dir, full.names = TRUE, no.. = TRUE)
  if (length(files) == 0) return(empty)

  info  <- file.info(files)
  bytes <- as.numeric(info$size)
  mtime <- info$mtime
  age   <- as.numeric(difftime(Sys.time(), mtime, units = "days"))

  out <- data.frame(
    file        = basename(files),
    extension   = tools::file_ext(files),
    size_bytes  = bytes,
    size_mb     = round(bytes / (1024 ^ 2), 3),
    modified    = mtime,
    age_days    = round(age, 2),
    path        = files,
    stringsAsFactors = FALSE
  )
  out <- out[order(-out$size_bytes), , drop = FALSE]
  rownames(out) <- NULL
  attr(out, "cache_dir") <- cache_dir
  out
}

test_that("hmrc_cache_info() returns empty data frame when cache is missing", {
  tmp <- file.path(tempdir(), paste0("hmrc-cache-info-", Sys.getpid(), "-empty"))
  unlink(tmp, recursive = TRUE)
  op <- options(hmrc.cache_dir = tmp)
  on.exit(options(op), add = TRUE)

  out <- hmrc_cache_info()
  expect_s3_class(out, "data.frame")
  expect_equal(nrow(out), 0L)
  expect_named(out, c("file", "extension", "size_bytes", "size_mb",
                      "modified", "age_days", "path"))
  expect_equal(attr(out, "cache_dir"), tmp)
})

test_that("hmrc_cache_info() lists cached files with sizes and ages", {
  tmp <- file.path(tempdir(), paste0("hmrc-cache-info-", Sys.getpid(), "-files"))
  dir.create(tmp, recursive = TRUE, showWarnings = FALSE)
  op <- options(hmrc.cache_dir = tmp)
  on.exit({
    options(op)
    unlink(tmp, recursive = TRUE)
  }, add = TRUE)

  writeLines("a", file.path(tmp, "1234567890_0010.ods"))
  writeLines("bb", file.path(tmp, "9876543210_0008.xlsx"))

  out <- hmrc_cache_info()
  expect_equal(nrow(out), 2L)
  expect_true(all(c("1234567890_0010.ods", "9876543210_0008.xlsx") %in% out$file))
  expect_true(all(out$size_bytes > 0))
  expect_true(all(out$age_days >= 0))
  expect_s3_class(out$modified, "POSIXt")
})

#' HMRC dataset catalogue
#'
#' A data frame describing every HMRC dataset known to the package, including
#' those exposed by an `hmrc_*` function and those on the development roadmap
#' (where `function_name` is `NA`). Use [hmrc_search()] for keyword search and
#' [hmrc_publications()] for a tidy index.
#'
#' @format A data frame with columns:
#'   \describe{
#'     \item{dataset}{Character. Short identifier.}
#'     \item{publication}{Character. GOV.UK publication name.}
#'     \item{function_name}{Character. Exporting function (or `NA`).}
#'     \item{frequency}{Character. `monthly`, `quarterly`, `annual`, etc.}
#'     \item{start}{Character. First period of the published series.}
#'     \item{publisher}{Character. Usually `"HMRC"`.}
#'     \item{slug}{Character. GOV.UK statistics slug.}
#'     \item{url}{Character. URL of the publication landing page.}
#'     \item{description}{Character. One-line description.}
#'     \item{tags}{Character. Space-separated keywords searched by
#'       [hmrc_search()].}
#'   }
"catalogue"

#' Search the HMRC dataset catalogue
#'
#' Fuzzy keyword search across publication name, description, tags, and
#' dataset identifier. Returns a tidy table of matching rows so users can
#' discover what the package exposes (and what is on the roadmap).
#'
#' @param query Character. Keyword or regex pattern. Case-insensitive.
#'   Matches anywhere in `dataset`, `publication`, `description`, or `tags`.
#'   If `NULL` or empty, the full catalogue is returned.
#' @param implemented Logical or `NULL`. If `TRUE`, only rows with a
#'   working `function_name` are returned. If `FALSE`, only roadmap rows.
#'   `NULL` (default) returns both.
#' @param frequency Optional character vector to filter by frequency
#'   (e.g. `"monthly"`, `"annual"`).
#' @return A data frame, sorted with implemented datasets first.
#'
#' @examples
#' # Everything mentioning VAT
#' hmrc_search("vat")
#'
#' # Only annual publications already implemented
#' hmrc_search(implemented = TRUE, frequency = "annual")
#'
#' # Roadmap items mentioning capital gains
#' hmrc_search("capital gains", implemented = FALSE)
#'
#' @family infrastructure
#' @export
hmrc_search <- function(query = NULL, implemented = NULL, frequency = NULL) {
  cat <- catalogue
  out <- cat

  if (!is.null(query)) {
    if (!is.character(query) || length(query) != 1L) {
      cli::cli_abort("{.arg query} must be a single character string.")
    }
    if (nzchar(query)) {
      hay <- paste(out$dataset, out$publication, out$description, out$tags,
                   sep = " | ")
      hits <- grepl(query, hay, ignore.case = TRUE, perl = TRUE)
      out  <- out[hits, , drop = FALSE]
    }
  }

  if (!is.null(implemented)) {
    if (!is.logical(implemented) || length(implemented) != 1L) {
      cli::cli_abort("{.arg implemented} must be `TRUE`, `FALSE`, or `NULL`.")
    }
    has_fn <- !is.na(out$function_name)
    out    <- out[if (implemented) has_fn else !has_fn, , drop = FALSE]
  }

  if (!is.null(frequency)) {
    bad <- setdiff(frequency, unique(cat$frequency))
    if (length(bad) > 0) {
      cli::cli_warn(c(
        "Frequenc{?y/ies} not in catalogue: {.val {bad}}",
        "i" = "Available: {.val {sort(unique(cat$frequency))}}"
      ))
    }
    out <- out[out$frequency %in% frequency, , drop = FALSE]
  }

  # Implemented rows first, then alphabetical
  ord <- order(is.na(out$function_name), out$dataset)
  out <- out[ord, c("dataset", "publication", "function_name",
                    "frequency", "start", "description"), drop = FALSE]
  rownames(out) <- NULL
  out
}

#' Index of HMRC publications known to the package
#'
#' Tidy roster of HMRC publications, marking which are exposed via an
#' `hmrc_*` function and which are on the development roadmap. Useful for
#' planning analyses, citing the package, and tracking coverage over time.
#'
#' @param status One of `"all"` (default), `"implemented"`, or `"planned"`.
#' @return A data frame with one row per publication and a `status` column
#'   (`"implemented"` or `"planned"`).
#'
#' @examples
#' # Everything in the catalogue
#' hmrc_publications()
#'
#' # Only roadmap items
#' hmrc_publications("planned")
#'
#' @family infrastructure
#' @export
hmrc_publications <- function(status = c("all", "implemented", "planned")) {
  status <- match.arg(status)

  out <- catalogue
  out$status <- ifelse(is.na(out$function_name), "planned", "implemented")

  if (status != "all") {
    out <- out[out$status == status, , drop = FALSE]
  }

  out <- out[, c("status", "dataset", "publication", "function_name",
                 "frequency", "start", "publisher", "url", "description"),
             drop = FALSE]
  ord <- order(out$status != "implemented", out$publication)
  out <- out[ord, , drop = FALSE]
  rownames(out) <- NULL
  out
}

test_that("catalogue dataset is exported and well-formed", {
  cat <- hmrc::catalogue
  expect_s3_class(cat, "data.frame")
  expect_named(cat, c("dataset", "publication", "function_name",
                      "frequency", "start", "publisher", "slug",
                      "url", "description", "tags"))
  expect_gt(nrow(cat), 10L)
  expect_false(any(duplicated(cat$dataset)))
  expect_true(any(!is.na(cat$function_name)))
})

test_that("hmrc_search() returns matching rows for keyword query", {
  # Word-boundary regex to avoid matching "vat" in "innovation" etc.
  out <- hmrc_search("\\bvat\\b")
  expect_s3_class(out, "data.frame")
  expect_gt(nrow(out), 0L)
  expect_true("vat_monthly" %in% out$dataset)
  hay <- paste(out$dataset, out$publication, out$description)
  expect_true(all(grepl("\\bvat\\b", hay, ignore.case = TRUE)))
})

test_that("hmrc_search() supports case-insensitive regex", {
  out <- hmrc_search("CAPITAL.*GAINS")
  expect_true(any(grepl("capital gains", out$description, ignore.case = TRUE)) ||
              any(grepl("capital_gains", out$dataset)))
})

test_that("hmrc_search() implemented filter works", {
  imp <- hmrc_search(implemented = TRUE)
  pln <- hmrc_search(implemented = FALSE)
  expect_true(all(!is.na(imp$function_name)))
  expect_true(all(is.na(pln$function_name)))
})

test_that("hmrc_search() frequency filter works", {
  out <- hmrc_search(frequency = "annual")
  expect_true(all(out$frequency %in% "annual"))
})

test_that("hmrc_search() rejects invalid query types", {
  expect_error(hmrc_search(query = 1), "character string")
  expect_error(hmrc_search(implemented = "yes"), "TRUE")
})

test_that("hmrc_publications() returns indexed publications", {
  out <- hmrc_publications()
  expect_s3_class(out, "data.frame")
  expect_true("status" %in% names(out))
  expect_true(all(out$status %in% c("implemented", "planned")))
  expect_true("hmrc_tax_receipts" %in% out$function_name)
})

test_that("hmrc_publications() status filter works", {
  imp <- hmrc_publications("implemented")
  pln <- hmrc_publications("planned")
  expect_true(all(imp$status == "implemented"))
  expect_true(all(pln$status == "planned"))
  expect_true(all(!is.na(imp$function_name)))
  expect_true(all(is.na(pln$function_name)))
})

test_that("hmrc_publications() rejects invalid status", {
  expect_error(hmrc_publications("nonsense"))
})

#' List available tax heads in the HMRC Tax Receipts bulletin
#'
#' Returns a data frame describing all tax and duty heads available in
#' [hmrc_tax_receipts()]. No network connection is required: the data is
#' bundled with the package.
#'
#' @return A data frame with columns:
#'   \describe{
#'     \item{tax_head}{Character. Identifier used in the `tax` argument of
#'       [hmrc_tax_receipts()].}
#'     \item{description}{Character. Plain-English description.}
#'     \item{category}{Character. Broad grouping.}
#'     \item{available_from}{Character. Earliest year of monthly data.}
#'   }
#'
#' @examples
#' hmrc_list_tax_heads()
#'
#' @family infrastructure
#' @export
hmrc_list_tax_heads <- function() {
  tax_heads
}

test_that("hmrc_list_tax_heads() returns a data frame with expected structure", {
  out <- hmrc_list_tax_heads()
  expect_s3_class(out, "data.frame")
  expect_named(out, c("tax_head", "description", "category", "available_from"))
  expect_true(nrow(out) > 0)
  expect_type(out$tax_head, "character")
  expect_type(out$description, "character")
  expect_type(out$category, "character")
})

test_that("hmrc_list_tax_heads() contains expected tax heads", {
  out <- hmrc_list_tax_heads()
  expect_true("income_tax" %in% out$tax_head)
  expect_true("vat" %in% out$tax_head)
  expect_true("nics_total" %in% out$tax_head)
  expect_true("total_receipts" %in% out$tax_head)
})

test_that("hmrc_list_tax_heads() has unique, non-missing identifiers", {
  out <- hmrc_list_tax_heads()
  expect_false(any(is.na(out$tax_head)))
  expect_equal(length(unique(out$tax_head)), nrow(out))
})

test_that("hmrc_list_tax_heads() categories come from the expected set", {
  out  <- hmrc_list_tax_heads()
  cats <- c("income", "nics", "consumption", "property", "environment",
            "expenditure", "other", "total")
  expect_true(all(out$category %in% cats))
})

#' Download HMRC stamp duty receipts
#'
#' Downloads and tidies the HMRC Annual Stamp Tax Statistics, covering
#' Stamp Duty Land Tax (SDLT), Stamp Duty Reserve Tax (SDRT) on shares,
#' and other stamp duties. Annual data from 2003-04 to the most recent
#' financial year, published each December.
#'
#' @param type Character vector or `NULL` (default = all types).
#'   Valid values: `"sdlt_property"`, `"sdlt_leases"`, `"sdlt_total"`,
#'   `"sdrt"`, `"stamp_duty"`, `"total"`.
#' @param cache Logical. Use cached file if available (default `TRUE`).
#'
#' @return An `hmrc_tbl` with columns `tax_year`, `type`, `description`,
#'   `receipts_gbp_m` (rounded to nearest GBP 5m by HMRC).
#'
#' @source
#' <https://www.gov.uk/government/statistics/uk-stamp-tax-statistics>
#'
#' @examples
#' \donttest{
#' op <- options(hmrc.cache_dir = tempdir())
#' hmrc_stamp_duty()
#' hmrc_stamp_duty(type = "sdlt_total")
#' options(op)
#' }
#'
#' @family data fetchers
#' @export
hmrc_stamp_duty <- function(type  = NULL,
                            cache = TRUE) {

  valid_types <- c("sdlt_property", "sdlt_leases", "sdlt_total",
                   "sdrt", "stamp_duty", "total")
  if (!is.null(type)) {
    bad <- setdiff(type, valid_types)
    if (length(bad) > 0) {
      cli::cli_abort(c(
        "Unknown type{?s}: {.val {bad}}",
        "i" = "Valid values: {.val {valid_types}}"
      ))
    }
  }

  slug <- "uk-stamp-tax-statistics"
  loc  <- resolve_govuk_attachment(slug, "Annual_Stamps")
  path <- download_cached(loc$attachment_url, cache = cache)

  cli::cli_progress_step("Parsing data")
  out <- parse_stamp_duty_table(path)

  if (!is.null(type)) out <- out[out$type %in% type, ]
  rownames(out) <- NULL

  cli::cli_progress_done()

  new_hmrc_tbl(
    out,
    dataset          = "stamp_duty_annual",
    hmrc_publication = "UK Stamp Tax statistics",
    source_url       = loc$page_url,
    attachment_url   = loc$attachment_url,
    slug             = slug,
    cell_methods     = "cash",
    frequency        = "annual"
  )
}

# ── Internal parser ────────────────────────────────────────────────────────────

STAMP_COLS <- c(
  "sdlt_property" = "SDLT on property (excl. new leases)",
  "sdlt_leases"   = "SDLT on new leases",
  "sdlt_total"    = "SDLT total",
  "sdrt"          = "Stamp Duty Reserve Tax (shares)",
  "stamp_duty"    = "Stamp Duty on documents",
  "total"         = "Total stamp taxes"
)

parse_stamp_duty_table <- function(path) {
  raw <- readODS::read_ods(path, sheet = "Table_1a",
                           col_names = FALSE, as_tibble = FALSE)

  data_rows  <- raw[8:nrow(raw), ]
  years_raw  <- trimws(as.character(data_rows[[1]]))
  valid      <- grepl("^\\d{4}-\\d{2}$", years_raw)
  years      <- years_raw[valid]
  data_rows  <- data_rows[valid, ]

  col_keys  <- names(STAMP_COLS)
  col_descs <- unname(STAMP_COLS)

  results <- list()
  for (j in seq_along(col_keys)) {
    col_idx <- j + 1
    if (col_idx > ncol(data_rows)) break
    vals <- suppressWarnings(as.numeric(as.character(data_rows[[col_idx]])))
    results[[j]] <- data.frame(
      tax_year       = years,
      type           = col_keys[j],
      description    = col_descs[j],
      receipts_gbp_m = vals,
      stringsAsFactors = FALSE
    )
  }

  out <- do.call(rbind, results)
  out <- out[order(out$type, out$tax_year), ]
  rownames(out) <- NULL
  out
}

test_that("hmrc_stamp_duty() rejects unknown type", {
  expect_error(
    hmrc_stamp_duty(type = "not_a_type"),
    "Unknown type"
  )
})

test_that("hmrc_stamp_duty() returns expected structure", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_stamp_duty(type = "sdlt_total")

  expect_s3_class(out, "hmrc_tbl")
  expect_named(out, c("tax_year", "type", "description", "receipts_gbp_m"))
  expect_true(all(out$type == "sdlt_total"))

  m <- hmrc_meta(out)
  expect_equal(m$dataset, "stamp_duty_annual")
  expect_equal(m$frequency, "annual")
})

# deprec.R — backwards-compatible aliases for the pre-0.4.0 `get_*` API.
#
# The package adopted the `hmrc_*` prefix in v0.4.0 to align with sibling
# packages (boe, fred, ons, obr). The old `get_*` names are kept as thin
# deprecated wrappers so existing user code continues to work; they will be
# removed in v0.6.0.
#
# Each alias issues a one-time-per-session deprecation warning via
# `lifecycle::deprecate_warn()` and forwards every argument unchanged.

#' Deprecated get_* aliases
#'
#' Renamed to use the `hmrc_*` prefix in v0.4.0. These aliases will be
#' removed in v0.6.0.
#'
#' @keywords internal
#' @name deprecated
NULL

#' @rdname deprecated
#' @export
get_tax_receipts <- function(...) {
  lifecycle::deprecate_warn("0.4.0", "get_tax_receipts()", "hmrc_tax_receipts()")
  hmrc_tax_receipts(...)
}

#' @rdname deprecated
#' @export
get_vat <- function(...) {
  lifecycle::deprecate_warn("0.4.0", "get_vat()", "hmrc_vat()")
  hmrc_vat(...)
}

#' @rdname deprecated
#' @export
get_fuel_duties <- function(...) {
  lifecycle::deprecate_warn("0.4.0", "get_fuel_duties()", "hmrc_fuel_duties()")
  hmrc_fuel_duties(...)
}

#' @rdname deprecated
#' @export
get_tobacco_duties <- function(...) {
  lifecycle::deprecate_warn("0.4.0", "get_tobacco_duties()", "hmrc_tobacco_duties()")
  hmrc_tobacco_duties(...)
}

#' @rdname deprecated
#' @export
get_corporation_tax <- function(...) {
  lifecycle::deprecate_warn("0.4.0", "get_corporation_tax()", "hmrc_corporation_tax()")
  hmrc_corporation_tax(...)
}

#' @rdname deprecated
#' @export
get_stamp_duty <- function(...) {
  lifecycle::deprecate_warn("0.4.0", "get_stamp_duty()", "hmrc_stamp_duty()")
  hmrc_stamp_duty(...)
}

#' @rdname deprecated
#' @export
get_property_transactions <- function(...) {
  lifecycle::deprecate_warn("0.4.0", "get_property_transactions()",
                            "hmrc_property_transactions()")
  hmrc_property_transactions(...)
}

#' @rdname deprecated
#' @export
get_rd_credits <- function(...) {
  lifecycle::deprecate_warn("0.4.0", "get_rd_credits()", "hmrc_rd_credits()")
  hmrc_rd_credits(...)
}

#' @rdname deprecated
#' @export
get_tax_gap <- function(...) {
  lifecycle::deprecate_warn("0.4.0", "get_tax_gap()", "hmrc_tax_gap()")
  hmrc_tax_gap(...)
}

#' @rdname deprecated
#' @export
get_income_tax_stats <- function(...) {
  lifecycle::deprecate_warn("0.4.0", "get_income_tax_stats()",
                            "hmrc_income_tax_stats()")
  hmrc_income_tax_stats(...)
}

#' @rdname deprecated
#' @export
list_tax_heads <- function() {
  lifecycle::deprecate_warn("0.4.0", "list_tax_heads()", "hmrc_list_tax_heads()")
  hmrc_list_tax_heads()
}

#' @rdname deprecated
#' @export
clear_cache <- function(max_age_days = NULL) {
  lifecycle::deprecate_warn("0.4.0", "clear_cache()", "hmrc_clear_cache()")
  hmrc_clear_cache(max_age_days = max_age_days)
}

test_that("get_* aliases warn and forward to hmrc_* equivalents", {
  withr::local_options(lifecycle_verbosity = "warning")

  expect_warning(out <- list_tax_heads(), "deprecated")
  expect_s3_class(out, "data.frame")
  expect_identical(out, hmrc_list_tax_heads())
})

test_that("clear_cache() alias warns and clears", {
  withr::local_options(lifecycle_verbosity = "warning")

  tmp <- file.path(tempdir(), paste0("hmrc-deprec-clear-", Sys.getpid()))
  unlink(tmp, recursive = TRUE)
  withr::local_options(hmrc.cache_dir = tmp)

  expect_warning(n <- clear_cache(), "deprecated")
  expect_type(n, "integer")
})

#' Download HMRC hydrocarbon oil (fuel duty) receipts
#'
#' Downloads and tidies the HMRC Hydrocarbon Oils Bulletin, which reports
#' monthly fuel duty receipts. Data runs from January 1990 to the most
#' recent published month, updated twice per year (January and July).
#'
#' @param fuel Character vector or `NULL` (default = all).
#'   Valid values: `"total"`, `"petrol"`, `"diesel"`, `"other"`.
#' @param start Character `"YYYY-MM"` or a `Date` object.
#' @param end Character `"YYYY-MM"` or a `Date` object.
#' @param cache Logical. Use cached file if available (default `TRUE`).
#'
#' @return An `hmrc_tbl` with columns `date`, `fuel`, `description`,
#'   `receipts_gbp_m`.
#'
#' @source
#' <https://www.gov.uk/government/statistics/hydrocarbon-oils-bulletin>
#'
#' @examples
#' \donttest{
#' op <- options(hmrc.cache_dir = tempdir())
#' hmrc_fuel_duties(fuel = "total", start = "2010-01")
#' hmrc_fuel_duties()
#' options(op)
#' }
#'
#' @family data fetchers
#' @export
hmrc_fuel_duties <- function(fuel  = NULL,
                             start = NULL,
                             end   = NULL,
                             cache = TRUE) {

  valid_fuels <- c("total", "petrol", "diesel", "other")
  if (!is.null(fuel)) {
    bad <- setdiff(fuel, valid_fuels)
    if (length(bad) > 0) {
      cli::cli_abort(c(
        "Unknown fuel category: {.val {bad}}",
        "i" = "Valid values: {.val {valid_fuels}}"
      ))
    }
  }
  if (!is.null(start)) parse_month_arg(start, "start")
  if (!is.null(end))   parse_month_arg(end,   "end")

  slug <- "hydrocarbon-oils-bulletin"
  loc  <- resolve_govuk_attachment(slug, "Oils_Tab")
  path <- download_cached(loc$attachment_url, cache = cache)

  cli::cli_progress_step("Parsing data")
  out <- parse_fuel_table(path)

  if (!is.null(fuel)) out <- out[out$fuel %in% fuel, ]
  out <- filter_dates(out, start, end)
  rownames(out) <- NULL

  cli::cli_progress_done()

  new_hmrc_tbl(
    out,
    dataset          = "fuel_duties_monthly",
    hmrc_publication = "Hydrocarbon Oils Bulletin (fuel duties)",
    source_url       = loc$page_url,
    attachment_url   = loc$attachment_url,
    slug             = slug,
    cell_methods     = "cash",
    frequency        = "monthly"
  )
}

# ── Internal parser ────────────────────────────────────────────────────────────

parse_fuel_table <- function(path) {
  raw <- readODS::read_ods(path, sheet = "Table_1_receipts",
                           col_names = FALSE, as_tibble = FALSE)

  monthly_start <- NA_integer_
  for (i in seq_len(nrow(raw))) {
    cell <- trimws(as.character(raw[i, 1]))
    if (grepl("^Month$|Table.*1c|by month", cell, ignore.case = TRUE)) {
      monthly_start <- i + 1
      break
    }
  }
  if (is.na(monthly_start)) {
    cli::cli_abort(
      "Could not locate monthly data in hydrocarbon oils bulletin. Please file an issue."
    )
  }

  data_rows <- raw[monthly_start:nrow(raw), ]

  dates_raw <- trimws(as.character(data_rows[[1]]))
  dates     <- suppressWarnings(
    as.Date(paste0("01 ", dates_raw), format = "%d %B %Y")
  )
  valid     <- !is.na(dates)
  dates     <- dates[valid]
  data_rows <- data_rows[valid, ]

  n_cols <- ncol(data_rows)

  safe_sum <- function(df, cols) {
    cols <- cols[cols <= ncol(df)]
    if (length(cols) == 0) return(rep(NA_real_, nrow(df)))
    vals <- suppressWarnings(
      apply(df[, cols, drop = FALSE], 2, function(x) as.numeric(as.character(x)))
    )
    if (is.null(dim(vals))) vals <- matrix(vals, ncol = 1)
    vals[is.na(vals)] <- 0
    rowSums(vals)
  }

  categories <- list(
    list(key = "petrol", desc = "Petrol duties",        cols = 2:8),
    list(key = "diesel", desc = "Diesel duties",        cols = 9:13),
    list(key = "other",  desc = "Other fuel duties",    cols = 14:16),
    list(key = "total",  desc = "Total oils receipts",  cols = n_cols)
  )

  results <- list()
  for (cat in categories) {
    vals <- if (length(cat$cols) == 1) {
      suppressWarnings(as.numeric(as.character(data_rows[[cat$cols]])))
    } else {
      safe_sum(data_rows, cat$cols)
    }
    results[[length(results) + 1]] <- data.frame(
      date           = dates,
      fuel           = cat$key,
      description    = cat$desc,
      receipts_gbp_m = vals,
      stringsAsFactors = FALSE
    )
  }

  out <- do.call(rbind, results)
  out <- out[order(out$fuel, out$date), ]
  rownames(out) <- NULL
  out
}

test_that("hmrc_fuel_duties() rejects unknown fuel", {
  expect_error(
    hmrc_fuel_duties(fuel = "rocket_fuel"),
    "Unknown fuel"
  )
})

test_that("hmrc_fuel_duties() returns expected structure", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_fuel_duties(fuel = "total", start = "2020-01", end = "2020-12")

  expect_s3_class(out, "hmrc_tbl")
  expect_named(out, c("date", "fuel", "description", "receipts_gbp_m"))
  expect_true(all(out$fuel == "total"))

  m <- hmrc_meta(out)
  expect_equal(m$dataset, "fuel_duties_monthly")
})

# utils-dates.R — shared date filtering helper

#' Filter a data frame by start/end month
#'
#' @param df Data frame with a `date` column (Date class).
#' @param start Character "YYYY-MM", Date, or NULL.
#' @param end   Character "YYYY-MM", Date, or NULL.
#' @return Filtered data frame.
#' @noRd
filter_dates <- function(df, start = NULL, end = NULL) {
  if (!is.null(start)) {
    start <- parse_month_arg(start, "start")
    df    <- df[df$date >= start, ]
  }
  if (!is.null(end)) {
    end <- parse_month_arg(end, "end")
    # Include the full end month
    end_last <- as.Date(format(end, "%Y-%m-01")) + 31
    end_last <- as.Date(format(end_last, "%Y-%m-01")) - 1
    df <- df[df$date <= end_last, ]
  }
  df
}

#' Parse a month argument to a Date (first of that month)
#' @noRd
parse_month_arg <- function(x, arg_name) {
  if (inherits(x, "Date")) return(as.Date(format(x, "%Y-%m-01")))
  if (is.character(x) && grepl("^\\d{4}-\\d{2}$", x)) {
    return(as.Date(paste0(x, "-01")))
  }
  cli::cli_abort(c(
    "{.arg {arg_name}} must be a {.cls Date} or a character string in {.val YYYY-MM} format.",
    "x" = "Got: {.val {x}}"
  ))
}

#' Download HMRC tax receipts and National Insurance contributions
#'
#' Downloads and tidies the monthly HMRC Tax Receipts and National Insurance
#' Contributions bulletin published on GOV.UK. The bulletin covers all major
#' UK taxes and duties from April 2008 to the most recent month (monthly
#' granularity), updated on approximately the 15th working day of each month.
#'
#' @param tax Character vector of tax head identifiers, or `NULL` (default)
#'   to return all available series. Use [hmrc_list_tax_heads()] to see valid
#'   values and descriptions.
#' @param start Character `"YYYY-MM"` or a `Date` object. If provided, rows
#'   before this month are dropped.
#' @param end Character `"YYYY-MM"` or a `Date` object. If provided, rows
#'   after this month are dropped.
#' @param cache Logical. If `TRUE` (default), the downloaded file is cached
#'   locally and reused on subsequent calls. Use [hmrc_clear_cache()] to
#'   reset.
#'
#' @return An `hmrc_tbl` (subclass of `data.frame`) with columns:
#'   \describe{
#'     \item{date}{`Date`. The first day of the reference month.}
#'     \item{tax_head}{Character. Tax or duty identifier (see
#'       [hmrc_list_tax_heads()]).}
#'     \item{description}{Character. Plain-English series label.}
#'     \item{receipts_gbp_m}{Numeric. Cash receipts in millions of pounds (GBP).}
#'   }
#'   Provenance metadata (source URL, fetch time, vintage) is attached as
#'   the `"hmrc_meta"` attribute and can be inspected with [hmrc_meta()].
#'
#' @source
#' <https://www.gov.uk/government/statistics/hmrc-tax-and-nics-receipts-for-the-uk>
#'
#' @examples
#' \donttest{
#' op <- options(hmrc.cache_dir = tempdir())
#' hmrc_tax_receipts()
#' hmrc_tax_receipts(tax = c("income_tax", "vat"))
#' hmrc_tax_receipts(start = "2020-04")
#' hmrc_tax_receipts(tax = "vat", start = "2019-01", end = "2024-12")
#' options(op)
#' }
#'
#' @family data fetchers
#' @export
hmrc_tax_receipts <- function(tax = NULL, start = NULL, end = NULL, cache = TRUE) {

  if (!is.null(tax)) {
    valid <- tax_heads$tax_head
    bad   <- setdiff(tax, valid)
    if (length(bad) > 0) {
      cli::cli_abort(c(
        "Unknown tax head{?s}: {.val {bad}}",
        "i" = "Run {.fn hmrc_list_tax_heads} to see valid values."
      ))
    }
  }
  if (!is.null(start)) parse_month_arg(start, "start")
  if (!is.null(end))   parse_month_arg(end,   "end")

  slug <- "hmrc-tax-and-nics-receipts-for-the-uk"
  loc  <- resolve_govuk_attachment(slug, "NS_Table\\.ods")
  path <- download_cached(loc$attachment_url, cache = cache)

  cli::cli_progress_step("Parsing data")
  raw <- parse_ns_table(path)

  if (!is.null(tax)) {
    raw <- raw[raw$tax_head %in% tax, ]
  }
  raw <- filter_dates(raw, start, end)
  rownames(raw) <- NULL

  cli::cli_progress_done()

  new_hmrc_tbl(
    raw,
    dataset          = "tax_receipts_monthly",
    hmrc_publication = "HMRC tax receipts and NICs (monthly bulletin)",
    source_url       = loc$page_url,
    attachment_url   = loc$attachment_url,
    slug             = slug,
    cell_methods     = "cash",
    frequency        = "monthly"
  )
}

# ── Internal: parse the NS_Table.ods ──────────────────────────────────────────

# Column header label → snake_case identifier mapping
# Only top-level series (no "(included within ...)" sub-components)
NS_COL_MAP <- c(
  "Total HMRC Receipts"               = "total_receipts",
  "Income Tax"                        = "income_tax",
  "Capital Gains Tax"                 = "capital_gains_tax",
  "Apprenticeship Levy"               = "apprenticeship_levy",
  "National Insurance Contributions"  = "nics_total",
  "PAYE NIC1 (EMP'er)"                = "nics_employer",
  "PAYE NIC1 (EMP'ee)"                = "nics_employee",
  "SA NIC2&4"                         = "nics_self_employed",
  "Value Added Tax"                   = "vat",
  "Corporation Tax"                   = "corporation_tax",
  "Bank Levy"                         = "bank_levy",
  "Bank Surcharge"                    = "bank_surcharge",
  "Diverted Profits Tax"              = "diverted_profits_tax",
  "Digital Services Tax"              = "digital_services_tax",
  "Residential Property Developer Tax"= "residential_property_developer_tax",
  "Energy Profits Levy"               = "energy_profits_levy",
  "Electricity Generators Levy"       = "electricity_generators_levy",
  "Economic Crime Levy"               = "economic_crime_levy",
  "Bank payroll tax"                  = "bank_payroll_tax",
  "Petroleum Revenue Tax"             = "petroleum_revenue_tax",
  "Hydrocarbon Oil (Fuel duties)"     = "fuel_duty",
  "Inheritance Tax"                   = "inheritance_tax",
  "Stamp Duty Shares"                 = "stamp_duty_shares",
  "Stamp Duty Land Tax"               = "sdlt",
  "Annual Tax on Enveloped Dwellings" = "ated",
  "Tobacco Duties"                    = "tobacco_duty",
  "Spirits Duties"                    = "spirits_duty",
  "Beer Duties"                       = "beer_duty",
  "Wines Duties"                      = "wine_duty",
  "Cider Duties"                      = "cider_duty",
  "Betting & Gaming"                  = "gambling_duties",
  "Air Passenger Duty"                = "air_passenger_duty",
  "Insurance Premium Tax"             = "insurance_premium_tax",
  "Landfill Tax"                      = "landfill_tax",
  "Climate Change Levy"               = "climate_change_levy",
  "Aggregates Levy"                   = "aggregates_levy",
  "Soft Drinks Industry Levy"         = "soft_drinks_levy",
  "Plastic Packaging Tax"             = "plastic_packaging_tax",
  "Customs Duties"                    = "customs_duties",
  "Misc"                              = "miscellaneous",
  "Penalties"                         = "penalties"
)

parse_ns_table <- function(path) {
  raw <- readODS::read_ods(path, sheet = "Receipts_Monthly",
                           col_names = FALSE, as_tibble = FALSE)

  header_row <- as.character(unlist(raw[6, ]))
  data_rows  <- raw[7:nrow(raw), ]

  dates_raw <- trimws(as.character(data_rows[[1]]))
  dates     <- suppressWarnings(
    as.Date(paste0("01 ", dates_raw), format = "%d %B %Y")
  )

  valid     <- !is.na(dates)
  dates     <- dates[valid]
  data_rows <- data_rows[valid, ]

  results <- list()

  for (j in seq_along(header_row)) {
    hdr <- trimws(header_row[j])
    if (j == 1) next
    if (grepl("included within", hdr, ignore.case = TRUE)) next
    if (!hdr %in% names(NS_COL_MAP)) next

    key  <- NS_COL_MAP[[hdr]]
    desc <- tax_heads$description[match(key, tax_heads$tax_head)]
    if (is.na(desc)) desc <- hdr

    vals <- suppressWarnings(as.numeric(as.character(data_rows[[j]])))

    results[[length(results) + 1]] <- data.frame(
      date           = dates,
      tax_head       = key,
      description    = desc,
      receipts_gbp_m = vals,
      stringsAsFactors = FALSE
    )
  }

  out <- do.call(rbind, results)
  out <- out[order(out$tax_head, out$date), ]
  rownames(out) <- NULL
  out
}

test_that("hmrc_tax_receipts() rejects unknown tax heads", {
  expect_error(
    hmrc_tax_receipts(tax = "not_a_tax"),
    "Unknown tax head"
  )
})

test_that("hmrc_tax_receipts() rejects invalid start format", {
  skip_on_cran()
  skip_if_offline()
  expect_error(
    hmrc_tax_receipts(start = "January 2020"),
    "YYYY-MM"
  )
})

test_that("hmrc_tax_receipts() rejects invalid end format", {
  skip_on_cran()
  skip_if_offline()
  expect_error(
    hmrc_tax_receipts(end = 2020),
    "YYYY-MM"
  )
})

test_that("hmrc_tax_receipts() returns expected structure with provenance", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_tax_receipts(tax = "vat", start = "2020-01", end = "2020-12")

  expect_s3_class(out, "hmrc_tbl")
  expect_s3_class(out, "data.frame")
  expect_named(out, c("date", "tax_head", "description", "receipts_gbp_m"))
  expect_s3_class(out$date, "Date")
  expect_type(out$tax_head, "character")
  expect_type(out$receipts_gbp_m, "double")

  m <- hmrc_meta(out)
  expect_equal(m$dataset, "tax_receipts_monthly")
  expect_equal(m$cell_methods, "cash")
  expect_equal(m$frequency, "monthly")
  expect_match(m$source_url, "hmrc-tax-and-nics-receipts")
})

test_that("hmrc_tax_receipts() date range filtering works", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_tax_receipts(tax = "income_tax", start = "2022-01", end = "2022-06")
  expect_true(all(out$date >= as.Date("2022-01-01")))
  expect_true(all(out$date <= as.Date("2022-06-30")))
})

test_that("hmrc_tax_receipts() tax filter returns only requested heads", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_tax_receipts(tax = c("vat", "income_tax"),
                           start = "2023-01", end = "2023-03")
  expect_true(all(out$tax_head %in% c("vat", "income_tax")))
})

#' Download monthly UK property transaction counts
#'
#' Downloads and tidies the HMRC Monthly Property Transactions bulletin,
#' which counts residential and non-residential property transactions
#' (SDLT returns, LBTT in Scotland, LTT in Wales) for England, Scotland,
#' Wales, Northern Ireland, and the UK total. Data runs from April 2005
#' to the most recent completed month.
#'
#' @param type Character. One of `"all"` (default), `"residential"`, or
#'   `"non_residential"`.
#' @param nation Character vector or `NULL` (default = all nations).
#'   Valid values: `"uk"`, `"england"`, `"scotland"`, `"wales"`,
#'   `"northern_ireland"`.
#' @param start Character `"YYYY-MM"` or a `Date` object.
#' @param end Character `"YYYY-MM"` or a `Date` object.
#' @param cache Logical. Use cached file if available (default `TRUE`).
#'
#' @return An `hmrc_tbl` with columns `date`, `nation`, `type`,
#'   `transactions`.
#'
#' @source
#' <https://www.gov.uk/government/statistics/monthly-property-transactions-completed-in-the-uk-with-value-40000-or-above>
#'
#' @examples
#' \donttest{
#' op <- options(hmrc.cache_dir = tempdir())
#' hmrc_property_transactions()
#' hmrc_property_transactions(type = "residential", nation = "england",
#'                            start = "2020-01")
#' options(op)
#' }
#'
#' @family data fetchers
#' @export
hmrc_property_transactions <- function(type   = c("all", "residential", "non_residential"),
                                       nation = NULL,
                                       start  = NULL,
                                       end    = NULL,
                                       cache  = TRUE) {

  type <- match.arg(type)

  if (!is.null(start)) parse_month_arg(start, "start")
  if (!is.null(end))   parse_month_arg(end,   "end")

  valid_nations <- c("uk", "england", "scotland", "wales", "northern_ireland")
  if (!is.null(nation)) {
    bad <- setdiff(tolower(nation), valid_nations)
    if (length(bad) > 0) {
      cli::cli_abort(c(
        "Unknown nation{?s}: {.val {bad}}",
        "i" = "Valid values: {.val {valid_nations}}"
      ))
    }
    nation <- tolower(nation)
  }

  slug <- "monthly-property-transactions-completed-in-the-uk-with-value-40000-or-above"
  loc  <- resolve_govuk_attachment(slug, "MPT_Tab")
  path <- download_cached(loc$attachment_url, cache = cache)

  cli::cli_progress_step("Parsing data")
  out <- parse_mpt_table(path)

  if (type != "all") {
    out <- out[out$type == type, ]
  }
  if (!is.null(nation)) {
    out <- out[out$nation %in% nation, ]
  }

  out <- filter_dates(out, start, end)
  rownames(out) <- NULL

  cli::cli_progress_done()

  new_hmrc_tbl(
    out,
    dataset          = "property_transactions_monthly",
    hmrc_publication = "Monthly UK property transactions",
    source_url       = loc$page_url,
    attachment_url   = loc$attachment_url,
    slug             = slug,
    cell_methods     = "counts",
    frequency        = "monthly"
  )
}

# ── Internal parser ────────────────────────────────────────────────────────────

MPT_SHEET_MAP <- c(
  residential     = "Residential_monthly",
  non_residential = "Non-residential_monthly"
)

MPT_NATION_COLS <- c(
  "England"          = "england",
  "Scotland"         = "scotland",
  "Wales"            = "wales",
  "Northern Ireland" = "northern_ireland",
  "UK"               = "uk"
)

parse_mpt_table <- function(path) {
  results <- list()

  for (type_key in names(MPT_SHEET_MAP)) {
    sheet <- MPT_SHEET_MAP[[type_key]]
    raw   <- readODS::read_ods(path, sheet = sheet,
                               col_names = FALSE, as_tibble = FALSE)

    header_row <- as.character(unlist(raw[6, ]))
    data_rows  <- raw[7:nrow(raw), ]

    dates_raw  <- trimws(as.character(data_rows[[1]]))
    dates      <- suppressWarnings(
      as.Date(paste0("01 ", dates_raw), format = "%d %B %Y")
    )

    valid     <- !is.na(dates)
    dates     <- dates[valid]
    data_rows <- data_rows[valid, ]

    for (j in seq_along(header_row)) {
      hdr <- trimws(header_row[j])
      if (!hdr %in% names(MPT_NATION_COLS)) next

      nation_key <- MPT_NATION_COLS[[hdr]]
      vals       <- suppressWarnings(as.numeric(as.character(data_rows[[j]])))

      results[[length(results) + 1]] <- data.frame(
        date         = dates,
        nation       = nation_key,
        type         = type_key,
        transactions = vals,
        stringsAsFactors = FALSE
      )
    }
  }

  out <- do.call(rbind, results)
  out <- out[order(out$type, out$nation, out$date), ]
  rownames(out) <- NULL
  out
}

test_that("hmrc_property_transactions() rejects unknown nation", {
  expect_error(
    hmrc_property_transactions(nation = "atlantis"),
    "Unknown nation"
  )
})

test_that("hmrc_property_transactions() returns expected structure", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_property_transactions(nation = "england", type = "residential",
                                    start = "2022-01", end = "2022-06")

  expect_s3_class(out, "hmrc_tbl")
  expect_named(out, c("date", "nation", "type", "transactions"))
  expect_true(all(out$nation == "england"))
  expect_true(all(out$type == "residential"))

  m <- hmrc_meta(out)
  expect_equal(m$cell_methods, "counts")
  expect_equal(m$frequency, "monthly")
})

test_that("hmrc_property_transactions() type=all returns both types", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_property_transactions(nation = "uk", start = "2022-01", end = "2022-03")
  expect_true(all(c("residential", "non_residential") %in% out$type))
})

#' hmrc: Download and Tidy HMRC Statistical Data
#'
#' Provides functions to download, parse, and tidy statistical data published
#' by HM Revenue and Customs (HMRC) on GOV.UK. Returns annotated `hmrc_tbl`
#' data frames with provenance metadata (source URL, fetch time, vintage,
#' cell methods) for reproducible fiscal research. File URLs are resolved at
#' runtime via the GOV.UK Content API, so data is always current.
#'
#' @section Data fetchers:
#' - [hmrc_tax_receipts()] — monthly tax receipts and NICs (April 2008+)
#' - [hmrc_property_transactions()] — monthly property transactions (April 2005+)
#' - [hmrc_income_tax_stats()] — annual Income Tax liabilities by income range
#' - [hmrc_tax_gap()] — annual tax gap estimates by tax type
#' - [hmrc_stamp_duty()] — annual stamp duty receipts (SDLT, SDRT)
#' - [hmrc_vat()] — monthly VAT receipts breakdown (April 1973+)
#' - [hmrc_rd_credits()] — annual R&D tax credit statistics (2000-01+)
#' - [hmrc_fuel_duties()] — monthly fuel duty receipts (January 1990+)
#' - [hmrc_tobacco_duties()] — monthly tobacco duty receipts (January 1991+)
#' - [hmrc_corporation_tax()] — annual Corporation Tax receipts by type
#' - [hmrc_capital_gains()] — annual CGT taxpayers, gains, liabilities (1987-88+)
#' - [hmrc_inheritance_tax()] — IHT estates and liabilities by net-estate band
#' - [hmrc_patent_box()] — annual Patent Box election counts and relief
#' - [hmrc_creative_industries()] — annual Film/HETV/Games/Theatre/etc reliefs
#'
#' @section Discovery and infrastructure:
#' - [hmrc_search()] — keyword search of the dataset catalogue
#' - [hmrc_publications()] — index of implemented + planned publications
#' - [hmrc_list_tax_heads()] — lookup table of tax-receipts identifiers
#' - [hmrc_cache_info()] — inspect locally cached files
#' - [hmrc_clear_cache()] — manage locally cached files
#' - [hmrc_meta()] — extract provenance metadata from any `hmrc_tbl` result
#'
#' @section Citation:
#' Run `citation("hmrc")` for the structured citation, or see the
#' `CITATION.cff` file at the package root for the GitHub citation widget.
#'
#' @section Data source:
#' All data is published by HMRC on GOV.UK under the Open Government Licence.
#' See <https://www.gov.uk/government/organisations/hm-revenue-customs/about/statistics>.
#'
#' @keywords internal
#' @concept HMRC
#' @concept UK tax
#' @concept tax revenue
#' @concept VAT
#' @concept income tax
#' @concept corporation tax
#' @concept stamp duty
#' @concept fuel duty
#' @concept tobacco duty
#' @concept government revenue
"_PACKAGE"

utils::globalVariables(c("tax_heads", "catalogue"))

# utils-govuk-api.R — internal helpers for GOV.UK Content API + caching

GOVUK_BASE              <- "https://www.gov.uk"
GOVUK_CONTENT_API       <- "https://www.gov.uk/api/content/government/statistics/"
GOVUK_PUBLICATION_PATH  <- "/government/statistics/"

#' Resolve the current download URL for a GOV.UK statistics attachment
#'
#' HMRC files are hosted on `assets.publishing.service.gov.uk` with a media
#' hash in the path that changes each publication cycle. This function queries
#' the GOV.UK Content API and returns a list with both the publication page
#' URL and the resolved attachment URL.
#'
#' @param slug GOV.UK statistics page slug.
#' @param filename_pattern Regex pattern to identify the correct attachment.
#' @return A list with `page_url`, `attachment_url`, `attachment_name`,
#'   `slug`, and (when present) `public_updated_at`.
#' @noRd
resolve_govuk_attachment <- function(slug, filename_pattern) {
  api_url <- paste0(GOVUK_CONTENT_API, slug)

  cli::cli_progress_step("Resolving download URL from GOV.UK Content API")

  resp <- tryCatch(
    httr2::request(api_url) |>
      httr2::req_user_agent("hmrc R package (https://github.com/charlescoverdale/hmrc)") |>
      httr2::req_throttle(rate = 5 / 10) |>
      httr2::req_timeout(30) |>
      httr2::req_retry(max_tries = 3) |>
      httr2::req_error(is_error = function(r) FALSE) |>
      httr2::req_perform(),
    error = function(e) {
      cli::cli_abort(c(
        "Could not reach GOV.UK Content API.",
        "i" = "Check your internet connection.",
        "x" = conditionMessage(e)
      ))
    }
  )

  if (httr2::resp_status(resp) != 200L) {
    cli::cli_abort(c(
      "GOV.UK Content API returned HTTP {httr2::resp_status(resp)}.",
      "i" = "The publication slug may have changed: {.val {slug}}"
    ))
  }

  body        <- httr2::resp_body_json(resp)
  attachments <- body$details$attachments

  if (is.null(attachments) || length(attachments) == 0) {
    cli::cli_abort("No attachments found for GOV.UK slug: {.val {slug}}")
  }

  matches <- Filter(
    function(a) grepl(filename_pattern, a$filename %||% "", ignore.case = TRUE),
    attachments
  )

  if (length(matches) == 0) {
    filenames <- vapply(attachments, function(a) a$filename %||% "(unnamed)", character(1))
    cli::cli_abort(c(
      "No attachment matching {.val {filename_pattern}} found.",
      "i" = "Available files: {.val {filenames}}"
    ))
  }

  page_url <- paste0(GOVUK_BASE, GOVUK_PUBLICATION_PATH, slug)

  list(
    slug              = slug,
    page_url          = page_url,
    attachment_url    = matches[[1]]$url,
    attachment_name   = matches[[1]]$filename %||% NA_character_,
    public_updated_at = body$public_updated_at %||% NA_character_
  )
}

#' Backwards-compatible wrapper returning just the attachment URL.
#' @noRd
resolve_govuk_url <- function(slug, filename_pattern) {
  resolve_govuk_attachment(slug, filename_pattern)$attachment_url
}

#' Get the cache directory, respecting the `hmrc.cache_dir` option
#' @noRd
hmrc_cache_dir <- function() {
  getOption("hmrc.cache_dir", default = tools::R_user_dir("hmrc", "cache"))
}

#' Compute the cache filename for a URL
#' @noRd
hmrc_cache_path <- function(url, vintage_key = NULL) {
  cache_dir <- hmrc_cache_dir()
  ext       <- tools::file_ext(url)
  ext       <- if (nzchar(ext)) paste0(".", ext) else ""
  hash      <- digest_url(if (is.null(vintage_key)) url else paste0(url, "@", vintage_key))
  file.path(cache_dir, paste0(hash, ext))
}

#' Download a file with local caching
#'
#' @param url URL to download.
#' @param cache Logical: use cached file if available?
#' @param vintage_key Optional string mixed into the cache key so vintage
#'   downloads do not collide with the latest version.
#' @return Path to the local (cached) file.
#' @noRd
download_cached <- function(url, cache = TRUE, vintage_key = NULL) {
  cache_dir  <- hmrc_cache_dir()
  cache_file <- hmrc_cache_path(url, vintage_key = vintage_key)

  if (cache && file.exists(cache_file)) {
    cli::cli_progress_step("Using cached file")
    return(cache_file)
  }

  dir.create(cache_dir, recursive = TRUE, showWarnings = FALSE)
  cli::cli_progress_step("Downloading data file")

  tryCatch(
    httr2::request(url) |>
      httr2::req_user_agent("hmrc R package (https://github.com/charlescoverdale/hmrc)") |>
      httr2::req_throttle(rate = 5 / 10) |>
      httr2::req_timeout(120) |>
      httr2::req_retry(max_tries = 3) |>
      httr2::req_perform(path = cache_file),
    error = function(e) {
      if (file.exists(cache_file)) unlink(cache_file)
      cli::cli_abort(c(
        "Download failed.",
        "x" = conditionMessage(e)
      ))
    }
  )

  cache_file
}

#' Simple URL hash for cache filenames (no extra dependencies)
#' @noRd
digest_url <- function(url) {
  chars    <- utf8ToInt(url)
  weights  <- seq_along(chars)
  checksum <- sum(as.numeric(chars) * weights) %% (2^31 - 1)
  sprintf("%010.0f_%04d", as.numeric(checksum), nchar(url) %% 10000L)
}

#' Null coalescing operator
#' @noRd
`%||%` <- function(x, y) if (is.null(x)) y else x

#' Download HMRC Patent Box relief statistics
#'
#' Downloads and tidies Table 1 of the HMRC Patent Box Relief Statistics:
#' annual number of companies electing into the Patent Box regime and the
#' total relief claimed (in GBP million). The Patent Box was introduced
#' from 1 April 2013; data runs from tax year 2013-14 to the most recent
#' published year (typically with a one-year provisional lag). Published
#' annually each September.
#'
#' @param tax_year Character vector or `NULL` (default = all years).
#'   Filter to specific tax years, e.g. `"2023-24"`.
#' @param cache Logical. Use cached file if available (default `TRUE`).
#'
#' @return An `hmrc_tbl` with columns:
#'   \describe{
#'     \item{tax_year}{Character. Tax year, e.g. `"2023-24"`.}
#'     \item{companies}{Numeric. Number of companies electing into the regime
#'       (rounded to nearest 5 by HMRC).}
#'     \item{relief_gbp_m}{Numeric. Total relief in millions of pounds.}
#'   }
#'
#' @source
#' <https://www.gov.uk/government/statistics/patent-box-reliefs-statistics>
#'
#' @examples
#' \donttest{
#' op <- options(hmrc.cache_dir = tempdir())
#' hmrc_patent_box()
#' hmrc_patent_box(tax_year = "2022-23")
#' options(op)
#' }
#'
#' @family data fetchers
#' @export
hmrc_patent_box <- function(tax_year = NULL,
                            cache    = TRUE) {

  slug <- "patent-box-reliefs-statistics"
  loc  <- resolve_govuk_attachment(slug, "PB_publication_tables")
  path <- download_cached(loc$attachment_url, cache = cache)

  cli::cli_progress_step("Parsing data")
  out <- parse_patent_box_table1(path)

  if (!is.null(tax_year)) {
    bad <- setdiff(tax_year, unique(out$tax_year))
    if (length(bad) > 0) {
      cli::cli_warn(c(
        "Tax year{?s} not found: {.val {bad}}",
        "i" = "Available years: {.val {unique(out$tax_year)}}"
      ))
    }
    out <- out[out$tax_year %in% tax_year, ]
  }
  rownames(out) <- NULL

  cli::cli_progress_done()

  new_hmrc_tbl(
    out,
    dataset          = "patent_box",
    hmrc_publication = "Patent Box reliefs statistics (Table 1)",
    source_url       = loc$page_url,
    attachment_url   = loc$attachment_url,
    slug             = slug,
    cell_methods     = "counts/cash",
    frequency        = "annual"
  )
}

# ── Internal parser ────────────────────────────────────────────────────────────

parse_patent_box_table1 <- function(path) {
  raw <- readODS::read_ods(path, sheet = "Table_1",
                           col_names = FALSE, as_tibble = FALSE)

  # Find header row: col 1 == "Year"
  hdr_row <- NA_integer_
  for (i in seq_len(nrow(raw))) {
    cell <- trimws(as.character(raw[i, 1]))
    if (identical(tolower(cell), "year")) {
      hdr_row <- i
      break
    }
  }
  if (is.na(hdr_row)) {
    cli::cli_abort("Could not locate Patent Box Table 1 header row.")
  }

  data_rows <- raw[(hdr_row + 1L):nrow(raw), ]
  yr_raw    <- trimws(as.character(data_rows[[1]]))
  is_year   <- grepl("^\\d{4}-\\d{2}$", yr_raw)
  data_rows <- data_rows[is_year, ]
  yr_raw    <- yr_raw[is_year]

  parse_num <- function(x) {
    x <- gsub(",", "", trimws(as.character(x)))
    x[grepl("\\[", x)] <- NA_character_
    suppressWarnings(as.numeric(x))
  }

  data.frame(
    tax_year     = yr_raw,
    companies    = parse_num(data_rows[[2]]),
    relief_gbp_m = parse_num(data_rows[[3]]),
    stringsAsFactors = FALSE
  )
}

test_that("hmrc_patent_box() returns expected structure", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_patent_box()

  expect_s3_class(out, "hmrc_tbl")
  expect_named(out, c("tax_year", "companies", "relief_gbp_m"))
  expect_true(all(grepl("^\\d{4}-\\d{2}$", out$tax_year)))
  expect_type(out$companies, "double")
  expect_type(out$relief_gbp_m, "double")

  m <- hmrc_meta(out)
  expect_equal(m$dataset, "patent_box")
  expect_equal(m$frequency, "annual")
})

test_that("hmrc_patent_box() series starts no later than 2014-15", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_patent_box()
  expect_true(min(out$tax_year) <= "2014-15")
})

test_that("hmrc_patent_box() warns on unknown year", {
  skip_on_cran()
  skip_if_offline()

  expect_warning(
    out <- hmrc_patent_box(tax_year = "1900-01"),
    "Tax year"
  )
  expect_equal(nrow(out), 0L)
})

#' Download HMRC Corporation Tax receipts by type
#'
#' Downloads and tidies the HMRC Corporation Tax Statistics annual
#' publication, reporting receipts broken down by tax type for the
#' most recent six financial years. Published annually in September.
#'
#' @param cache Logical. Use cached file if available (default `TRUE`).
#'
#' @return An `hmrc_tbl` with columns `tax_year`, `type`, `description`,
#'   `receipts_gbp_m`.
#'
#' @details
#' Some levies (e.g. Residential Property Developer Tax, Electricity
#' Generators Levy) were introduced mid-series and have `NA` values for
#' earlier years.
#'
#' @source
#' <https://www.gov.uk/government/collections/analyses-of-corporation-tax-receipts-and-liabilities>
#'
#' @examples
#' \donttest{
#' op <- options(hmrc.cache_dir = tempdir())
#' hmrc_corporation_tax()
#' options(op)
#' }
#'
#' @family data fetchers
#' @export
hmrc_corporation_tax <- function(cache = TRUE) {

  current_year <- as.integer(format(Sys.Date(), "%Y"))
  loc <- NULL
  for (yr in seq(current_year, current_year - 3L)) {
    slug <- paste0("corporation-tax-statistics-", yr)
    loc <- tryCatch(
      resolve_govuk_attachment(slug, "Corporation_Tax"),
      error = function(e) NULL
    )
    if (!is.null(loc)) break
  }
  if (is.null(loc)) {
    cli::cli_abort(c(
      "Could not find Corporation Tax statistics on GOV.UK.",
      "i" = "Tried slugs for {current_year} back to {current_year - 3L}.",
      "i" = "Check {.url https://www.gov.uk/government/collections/analyses-of-corporation-tax-receipts-and-liabilities}"
    ))
  }
  path <- download_cached(loc$attachment_url, cache = cache)

  cli::cli_progress_step("Parsing data")
  out <- parse_corporation_tax_table(path)
  rownames(out) <- NULL

  cli::cli_progress_done()

  new_hmrc_tbl(
    out,
    dataset          = "corporation_tax_annual",
    hmrc_publication = "Corporation Tax statistics",
    source_url       = loc$page_url,
    attachment_url   = loc$attachment_url,
    slug             = loc$slug,
    cell_methods     = "cash",
    frequency        = "annual"
  )
}

# ── Internal parser ────────────────────────────────────────────────────────────

parse_corporation_tax_table <- function(path) {
  raw <- readODS::read_ods(path, sheet = "Table_1A",
                           col_names = FALSE, as_tibble = FALSE)

  header_row <- as.character(unlist(raw[5, ]))
  year_cols  <- which(grepl("^20\\d{2}-\\d{2}$", trimws(header_row)))

  if (length(year_cols) == 0) {
    cli::cli_abort("Could not locate year columns in Corporation Tax table. Please file an issue.")
  }

  years <- trimws(header_row[year_cols])

  data_rows <- raw[6:nrow(raw), ]
  type_raw  <- trimws(as.character(data_rows[[1]]))
  valid     <- nzchar(type_raw) & !grepl("End of|^NA$", type_raw)
  data_rows <- data_rows[valid, ]
  type_raw  <- type_raw[valid]

  classify_type <- function(lbl) {
    l <- tolower(lbl)
    if (grepl("all corporate", l))                                    return("all_corporate_taxes")
    if (grepl("total.*onshore.*offshore|total.*corporation", l))      return("total_ct")
    if (grepl("onshore", l))                                          return("onshore_ct")
    if (grepl("offshore", l))                                         return("offshore_ct")
    if (grepl("bank levy|^bl$", l))                                   return("bank_levy")
    if (grepl("bank surcharge|^bs$", l))                              return("bank_surcharge")
    if (grepl("residential property developer|rpdt", l))              return("rpdt")
    if (grepl("energy profits levy|epl", l))                          return("energy_profits_levy")
    if (grepl("electricity generator", l))                            return("electricity_generators_levy")
    tolower(gsub("[^a-zA-Z0-9]+", "_", lbl))
  }

  results <- list()
  for (i in seq_len(nrow(data_rows))) {
    lbl <- type_raw[i]
    key <- classify_type(lbl)

    for (j in seq_along(year_cols)) {
      col_idx <- year_cols[j]
      if (col_idx > ncol(data_rows)) next
      val <- suppressWarnings(as.numeric(as.character(data_rows[i, col_idx])))
      results[[length(results) + 1]] <- data.frame(
        tax_year       = years[j],
        type           = key,
        description    = lbl,
        receipts_gbp_m = val,
        stringsAsFactors = FALSE
      )
    }
  }

  out <- do.call(rbind, results)
  out <- out[order(out$type, out$tax_year), ]
  rownames(out) <- NULL
  out
}

test_that("hmrc_corporation_tax() returns expected structure with provenance", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_corporation_tax()

  expect_s3_class(out, "hmrc_tbl")
  expect_named(out, c("tax_year", "type", "description", "receipts_gbp_m"))
  expect_type(out$tax_year, "character")
  expect_type(out$type, "character")
  expect_type(out$receipts_gbp_m, "double")

  m <- hmrc_meta(out)
  expect_equal(m$dataset, "corporation_tax_annual")
  expect_equal(m$frequency, "annual")
})

test_that("hmrc_corporation_tax() year format is YYYY-YY", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_corporation_tax()
  expect_true(all(grepl("^\\d{4}-\\d{2}$", out$tax_year)))
})

#' Download HMRC Inheritance Tax liabilities by estate band
#'
#' Downloads and tidies HMRC Table 12.1a of the Inheritance Tax Liabilities
#' Statistics: numbers of estates, tax due, average tax, and average
#' effective tax rate, broken down by net-estate band, for the latest
#' published year of death. Published annually in July, ~3 years after the
#' year of death due to the administration window.
#'
#' @param cache Logical. Use cached file if available (default `TRUE`).
#'
#' @return An `hmrc_tbl` (long format) with columns:
#'   \describe{
#'     \item{tax_year}{Character. Year of death, e.g. `"2022-23"`.}
#'     \item{estate_band_lower_gbp}{Numeric. Lower limit of the net-estate
#'       band in pounds.}
#'     \item{estate_band}{Character. Plain-English label, e.g.
#'       `"GBP 0-100k"`, `"Total"`.}
#'     \item{measure}{Character. One of `"number_not_taxed"`,
#'       `"number_taxed"`, `"tax_due_gbp_m"`, `"avg_tax_gbp"`,
#'       `"effective_rate_pct"`.}
#'     \item{value}{Numeric. Value, with `[z]` (no tax due to NRB) and
#'       `[c]` (disclosure suppression) returned as `NA`.}
#'   }
#'
#' @source
#' <https://www.gov.uk/government/statistics/inheritance-tax-liabilities-statistics>
#'
#' @examples
#' \donttest{
#' op <- options(hmrc.cache_dir = tempdir())
#' hmrc_inheritance_tax()
#' options(op)
#' }
#'
#' @family data fetchers
#' @export
hmrc_inheritance_tax <- function(cache = TRUE) {

  slug <- "inheritance-tax-liabilities-statistics"
  loc  <- resolve_govuk_attachment(slug, "table_12_1\\.ods")
  path <- download_cached(loc$attachment_url, cache = cache)

  cli::cli_progress_step("Parsing data")
  out <- parse_iht_12_1a(path)
  rownames(out) <- NULL

  cli::cli_progress_done()

  new_hmrc_tbl(
    out,
    dataset          = "inheritance_tax_annual",
    hmrc_publication = "Inheritance Tax liabilities statistics (Table 12.1a)",
    source_url       = loc$page_url,
    attachment_url   = loc$attachment_url,
    slug             = slug,
    cell_methods     = "liabilities",
    frequency        = "annual"
  )
}

# ── Internal parser ────────────────────────────────────────────────────────────
#
# IHT 12_1a layout (annual cross-section, ~3yr publication lag):
#   Sheet "12_1a"
#   Row 5 = headers, e.g. "Net estate (lower limits) [Note 1]" |
#           "2022 to 2023 number not taxed" | "2022 to 2023 number taxed" |
#           "2022 to 2023 tax due (£ million) [Note 2]" |
#           "2022 to 2023 average tax (£)" |
#           "2022 to 2023 average effective tax rate of taxpaying estates (per cent)"
#   Row 6+ = data: net-estate lower-limit GBP (0, 100000, ..., 10000000) +
#                  totals row "Total estates notified".
# Suppressed cells: "[z]" = no tax (estate below NRB), "[c]" = disclosure.

IHT_MEASURE_PATTERNS <- c(
  number_not_taxed   = "number not taxed",
  number_taxed       = "number taxed",
  tax_due_gbp_m      = "tax due",
  avg_tax_gbp        = "average tax",
  effective_rate_pct = "average effective tax rate"
)

parse_iht_12_1a <- function(path) {
  raw <- readODS::read_ods(path, sheet = "12_1a",
                           col_names = FALSE, as_tibble = FALSE)

  # Find header row: cell A* contains "Net estate (lower limits)"
  hdr_row <- NA_integer_
  for (i in seq_len(nrow(raw))) {
    cell <- trimws(as.character(raw[i, 1]))
    if (grepl("^Net estate", cell, ignore.case = TRUE)) {
      hdr_row <- i
      break
    }
  }
  if (is.na(hdr_row)) {
    cli::cli_abort("Could not locate IHT Table 12.1a header row. Please file an issue.")
  }

  header_row <- as.character(unlist(raw[hdr_row, ]))

  # Extract tax_year from any of the value-column headers
  tax_year <- NA_character_
  for (h in header_row[-1L]) {
    if (is.na(h)) next
    m <- regmatches(h, regexpr("\\d{4}\\s+to\\s+\\d{4}", h))
    if (length(m) == 1L && nzchar(m)) {
      yrs <- regmatches(m, gregexpr("\\d{4}", m))[[1]]
      tax_year <- paste0(yrs[1], "-", substr(yrs[2], 3, 4))
      break
    }
  }
  if (is.na(tax_year)) tax_year <- "unknown"

  # Map each value column to a measure key by header pattern
  col_measure <- rep(NA_character_, length(header_row))
  for (j in seq_along(header_row)) {
    h <- tolower(header_row[j] %||% "")
    for (key in names(IHT_MEASURE_PATTERNS)) {
      if (grepl(IHT_MEASURE_PATTERNS[[key]], h)) {
        col_measure[j] <- key
        break
      }
    }
  }

  data_rows <- raw[(hdr_row + 1L):nrow(raw), ]
  band_raw  <- trimws(as.character(data_rows[[1]]))

  total_idx <- grep("^Total estates", band_raw, ignore.case = TRUE)
  numeric_idx <- grep("^[0-9]", band_raw)
  keep_idx <- sort(unique(c(numeric_idx, total_idx)))
  if (length(keep_idx) == 0) {
    cli::cli_abort("No data rows found in IHT 12_1a.")
  }
  data_rows <- data_rows[keep_idx, ]
  band_raw  <- band_raw[keep_idx]

  is_total <- grepl("^Total", band_raw, ignore.case = TRUE)
  estate_band_lower_gbp <- suppressWarnings(as.numeric(band_raw))
  # Total row: keep NA lower limit, label "Total"
  estate_band_lower_gbp[is_total] <- NA_real_

  fmt_band <- function(low, total) {
    if (total) return("Total")
    if (is.na(low)) return(NA_character_)
    if (low == 0)       return("GBP 0-100k")
    if (low ==   100000) return("GBP 100k-200k")
    if (low ==   200000) return("GBP 200k-300k")
    if (low ==   300000) return("GBP 300k-400k")
    if (low ==   400000) return("GBP 400k-500k")
    if (low ==   500000) return("GBP 500k-600k")
    if (low ==   600000) return("GBP 600k-700k")
    if (low ==   700000) return("GBP 700k-800k")
    if (low ==   800000) return("GBP 800k-900k")
    if (low ==   900000) return("GBP 900k-1m")
    if (low ==  1000000) return("GBP 1m-1.5m")
    if (low ==  1500000) return("GBP 1.5m-2m")
    if (low ==  2000000) return("GBP 2m-3m")
    if (low ==  3000000) return("GBP 3m-4m")
    if (low ==  4000000) return("GBP 4m-5m")
    if (low ==  5000000) return("GBP 5m-7.5m")
    if (low ==  7500000) return("GBP 7.5m-10m")
    if (low == 10000000) return("GBP 10m+")
    paste0("GBP ", format(low, big.mark = ","), "+")
  }
  estate_band <- mapply(fmt_band, estate_band_lower_gbp, is_total,
                        USE.NAMES = FALSE)

  parse_num <- function(x) {
    x <- gsub(",", "", trimws(as.character(x)))
    x[grepl("\\[", x)] <- NA_character_
    suppressWarnings(as.numeric(x))
  }

  results <- list()
  for (j in seq_along(col_measure)) {
    if (is.na(col_measure[j])) next
    if (j > ncol(data_rows)) next
    results[[length(results) + 1L]] <- data.frame(
      tax_year              = tax_year,
      estate_band_lower_gbp = estate_band_lower_gbp,
      estate_band           = estate_band,
      measure               = col_measure[j],
      value                 = parse_num(data_rows[[j]]),
      stringsAsFactors      = FALSE
    )
  }

  if (length(results) == 0) {
    cli::cli_abort("No measure columns recognised in IHT 12_1a.")
  }

  out <- do.call(rbind, results)
  ord <- order(
    is.na(out$estate_band_lower_gbp),
    out$estate_band_lower_gbp,
    out$measure
  )
  out <- out[ord, ]
  rownames(out) <- NULL
  out
}

test_that("hmrc_inheritance_tax() returns expected structure", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_inheritance_tax()

  expect_s3_class(out, "hmrc_tbl")
  expect_named(out, c("tax_year", "estate_band_lower_gbp", "estate_band",
                      "measure", "value"))
  expect_true(all(grepl("^\\d{4}-\\d{2}$", out$tax_year)) ||
              all(out$tax_year == "unknown"))
  expect_true(any(out$estate_band == "Total"))
  expect_true(any(grepl("^GBP", out$estate_band)))

  m <- hmrc_meta(out)
  expect_equal(m$dataset, "inheritance_tax_annual")
  expect_equal(m$cell_methods, "liabilities")
})

test_that("hmrc_inheritance_tax() includes the five expected measures", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_inheritance_tax()
  expect_setequal(
    unique(out$measure),
    c("number_not_taxed", "number_taxed", "tax_due_gbp_m",
      "avg_tax_gbp", "effective_rate_pct")
  )
})

#' Tax head lookup table
#'
#' A data frame describing all tax and duty series available in
#' [hmrc_tax_receipts()].
#'
#' @format A data frame with 41 rows and 4 columns:
#'   \describe{
#'     \item{tax_head}{Character. Identifier used in the `tax` argument of
#'       [hmrc_tax_receipts()].}
#'     \item{description}{Character. Plain-English description.}
#'     \item{category}{Character. Broad grouping: `"income"`, `"nics"`,
#'       `"consumption"`, `"property"`, `"environment"`, `"expenditure"`,
#'       `"other"`, or `"total"`.}
#'     \item{available_from}{Character. Approximate start year of monthly data.}
#'   }
#'
#' @source
#' Derived from the HMRC Tax Receipts and NICs bulletin.
#' <https://www.gov.uk/government/statistics/hmrc-tax-and-nics-receipts-for-the-uk>
"tax_heads"

#' Download HMRC R&D tax credit statistics
#'
#' Downloads and tidies the HMRC Research and Development Tax Credits
#' Statistics publication, covering the SME R&D Relief and Research and
#' Development Expenditure Credit (RDEC) schemes. Annual data runs from
#' 2000-01 to the most recent financial year, published annually in September.
#'
#' @param scheme Character vector or `NULL` (default = all schemes).
#'   Valid values: `"sme"`, `"rdec"`, `"total"`.
#' @param measure Character vector or `NULL` (default = all measures).
#'   Valid values: `"claims"`, `"amount_gbp_m"`.
#' @param cache Logical. Use cached file if available (default `TRUE`).
#'
#' @return An `hmrc_tbl` with columns `tax_year`, `scheme`, `description`,
#'   `measure`, `value`.
#'
#' @details
#' Data before 2003-04 covers only the SME scheme (RDEC was introduced in
#' 2002). Figures for the most recent two years are provisional and subject
#' to revision as late claims are processed.
#'
#' @source
#' <https://www.gov.uk/government/statistics/corporate-tax-research-and-development-tax-credit>
#'
#' @examples
#' \donttest{
#' op <- options(hmrc.cache_dir = tempdir())
#' hmrc_rd_credits()
#' hmrc_rd_credits(scheme = "sme", measure = "claims")
#' options(op)
#' }
#'
#' @family data fetchers
#' @export
hmrc_rd_credits <- function(scheme  = NULL,
                            measure = NULL,
                            cache   = TRUE) {

  valid_schemes  <- c("sme", "rdec", "total")
  valid_measures <- c("claims", "amount_gbp_m")

  if (!is.null(scheme)) {
    bad <- setdiff(scheme, valid_schemes)
    if (length(bad) > 0) {
      cli::cli_abort(c(
        "Unknown scheme{?s}: {.val {bad}}",
        "i" = "Valid values: {.val {valid_schemes}}"
      ))
    }
  }
  if (!is.null(measure)) {
    bad <- setdiff(measure, valid_measures)
    if (length(bad) > 0) {
      cli::cli_abort(c(
        "Unknown measure{?s}: {.val {bad}}",
        "i" = "Valid values: {.val {valid_measures}}"
      ))
    }
  }

  slug <- "corporate-tax-research-and-development-tax-credit"
  loc  <- resolve_govuk_attachment(slug, "rd_tax_credits_main")
  path <- download_cached(loc$attachment_url, cache = cache)

  cli::cli_progress_step("Parsing data")
  out <- parse_rd_credits_table(path)

  if (!is.null(scheme))  out <- out[out$scheme  %in% scheme,  ]
  if (!is.null(measure)) out <- out[out$measure %in% measure, ]
  rownames(out) <- NULL

  cli::cli_progress_done()

  new_hmrc_tbl(
    out,
    dataset          = "rd_credits_annual",
    hmrc_publication = "R&D Tax Credits Statistics",
    source_url       = loc$page_url,
    attachment_url   = loc$attachment_url,
    slug             = slug,
    cell_methods     = "counts/cash",
    frequency        = "annual"
  )
}

# -- Internal parser -----------------------------------------------------------

parse_rd_credits_table <- function(path) {
  read_sheet <- function(sheet, sme_col, rdec_col, total_col) {
    raw <- readODS::read_ods(path, sheet = sheet,
                             col_names = FALSE, as_tibble = FALSE)
    data_rows <- raw[6:nrow(raw), ]
    years_raw <- trimws(as.character(data_rows[[1]]))
    valid     <- grepl("^20\\d{2}-\\d{2}$", years_raw)
    list(
      years     = years_raw[valid],
      data_rows = data_rows[valid, ],
      sme_col   = sme_col,
      rdec_col  = rdec_col,
      total_col = total_col
    )
  }

  rd1 <- read_sheet("RD1", sme_col = 7L, rdec_col = 11L, total_col = 13L)
  rd2 <- read_sheet("RD2", sme_col = 5L, rdec_col = 9L,  total_col = 11L)

  if (length(rd1$years) == 0) {
    cli::cli_abort("Could not locate year rows in R&D credits table. Please file an issue.")
  }

  extract <- function(sheet_data, col, scheme, desc, measure) {
    if (col > ncol(sheet_data$data_rows)) return(NULL)
    raw_vals <- as.character(sheet_data$data_rows[[col]])
    raw_vals <- gsub(",", "", raw_vals)
    vals <- suppressWarnings(as.numeric(raw_vals))
    data.frame(
      tax_year    = sheet_data$years,
      scheme      = scheme,
      description = desc,
      measure     = measure,
      value       = vals,
      stringsAsFactors = FALSE
    )
  }

  results <- list(
    extract(rd1, rd1$sme_col,   "sme",   "SME R&D Relief",                "claims"),
    extract(rd1, rd1$rdec_col,  "rdec",  "R&D Expenditure Credit (RDEC)", "claims"),
    extract(rd1, rd1$total_col, "total", "All R&D schemes",               "claims"),
    extract(rd2, rd2$sme_col,   "sme",   "SME R&D Relief",                "amount_gbp_m"),
    extract(rd2, rd2$rdec_col,  "rdec",  "R&D Expenditure Credit (RDEC)", "amount_gbp_m"),
    extract(rd2, rd2$total_col, "total", "All R&D schemes",               "amount_gbp_m")
  )
  results <- Filter(Negate(is.null), results)

  out <- do.call(rbind, results)
  out <- out[order(out$scheme, out$measure, out$tax_year), ]
  rownames(out) <- NULL
  out
}

test_that("hmrc_rd_credits() rejects unknown scheme/measure", {
  expect_error(
    hmrc_rd_credits(scheme = "huge"),
    "Unknown scheme"
  )
  expect_error(
    hmrc_rd_credits(measure = "frobnicators"),
    "Unknown measure"
  )
})

test_that("hmrc_rd_credits() returns expected structure", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_rd_credits(scheme = "sme", measure = "claims")

  expect_s3_class(out, "hmrc_tbl")
  expect_named(out, c("tax_year", "scheme", "description", "measure", "value"))
  expect_true(all(out$scheme == "sme"))
  expect_true(all(out$measure == "claims"))

  m <- hmrc_meta(out)
  expect_equal(m$dataset, "rd_credits_annual")
})

# hmrc-tbl.R — S3 class with provenance metadata

#' Construct an `hmrc_tbl` object
#'
#' Internal constructor used by every `hmrc_*` data fetcher to attach
#' provenance metadata to the returned data frame. Not exported.
#'
#' @param x A data frame holding the parsed dataset.
#' @param dataset Short identifier (matches a row of [hmrc_publications()]).
#' @param hmrc_publication Human-readable publication name.
#' @param source_url URL of the GOV.UK publication page.
#' @param attachment_url Direct URL of the parsed attachment.
#' @param slug GOV.UK statistics slug.
#' @param cell_methods Either `"cash"`, `"accruals"`, `"liabilities"`, or
#'   `"counts"`. Helps downstream users avoid mixing apples and oranges.
#' @param frequency `"monthly"`, `"quarterly"`, `"annual"`, or `"point_in_time"`.
#' @param vintage_date `Date` of the as-of vintage; `NA` for the latest
#'   published version.
#' @return An `hmrc_tbl` (subclass of `data.frame`).
#' @noRd
new_hmrc_tbl <- function(x,
                         dataset          = NA_character_,
                         hmrc_publication = NA_character_,
                         source_url       = NA_character_,
                         attachment_url   = NA_character_,
                         slug             = NA_character_,
                         cell_methods     = NA_character_,
                         frequency        = NA_character_,
                         vintage_date     = as.Date(NA)) {
  stopifnot(is.data.frame(x))

  attr(x, "hmrc_meta") <- list(
    dataset          = dataset,
    hmrc_publication = hmrc_publication,
    source_url       = source_url,
    attachment_url   = attachment_url,
    slug             = slug,
    cell_methods     = cell_methods,
    frequency        = frequency,
    vintage_date     = vintage_date,
    fetched_at       = Sys.time(),
    package_version  = as.character(utils::packageVersion("hmrc"))
  )

  if (!inherits(x, "hmrc_tbl")) {
    class(x) <- c("hmrc_tbl", class(x))
  }
  x
}

#' Provenance metadata of an `hmrc_tbl`
#'
#' Returns the provenance list attached to a result from any `hmrc_*` data
#' fetcher: source URL, fetch time, vintage, cell methods, and so on. Useful
#' for citation, audit trails, and reproducibility.
#'
#' @param x An object returned by an `hmrc_*` data function.
#' @return A named list, or `NULL` if `x` has no metadata.
#'
#' @examples
#' \donttest{
#' op <- options(hmrc.cache_dir = tempdir())
#' tr <- hmrc_tax_receipts(tax = "vat", start = "2024-01")
#' hmrc_meta(tr)
#' options(op)
#' }
#'
#' @family infrastructure
#' @export
hmrc_meta <- function(x) {
  attr(x, "hmrc_meta")
}

#' @export
print.hmrc_tbl <- function(x, ...) {
  m <- attr(x, "hmrc_meta")
  if (!is.null(m)) {
    pub <- m$hmrc_publication %||% "HMRC dataset"
    cat("# ", pub, "\n", sep = "")
    if (!is.na(m$source_url) && nzchar(m$source_url)) {
      cat("# Source: ", m$source_url, "\n", sep = "")
    }
    fetched <- if (inherits(m$fetched_at, "POSIXt")) {
      format(m$fetched_at, "%Y-%m-%d %H:%M:%S %Z", tz = "UTC")
    } else {
      "unknown"
    }
    vintage <- if (is.na(m$vintage_date)) "latest" else as.character(m$vintage_date)
    bits <- c(
      paste0("Fetched ", fetched),
      paste0("Vintage: ", vintage),
      if (!is.na(m$cell_methods)) paste0("Cells: ", m$cell_methods),
      if (!is.na(m$frequency))    paste0("Freq: ",  m$frequency),
      paste0(format(nrow(x), big.mark = ","), " rows x ", ncol(x), " cols")
    )
    cat("# ", paste(bits, collapse = " | "), "\n", sep = "")
    cat("\n")
  }
  NextMethod()
}

#' @export
format.hmrc_tbl <- function(x, ...) {
  NextMethod()
}

#' @export
as.data.frame.hmrc_tbl <- function(x, ...) {
  attr(x, "hmrc_meta") <- NULL
  class(x) <- setdiff(class(x), "hmrc_tbl")
  x
}

# Subsetting drops to plain data.frame if rows or columns are removed —
# provenance no longer reflects the full dataset.
#' @export
`[.hmrc_tbl` <- function(x, i, j, ..., drop = TRUE) {
  out <- NextMethod()
  if (is.data.frame(out)) {
    attr(out, "hmrc_meta") <- attr(x, "hmrc_meta")
    if (!inherits(out, "hmrc_tbl")) class(out) <- c("hmrc_tbl", class(out))
  }
  out
}

test_that("hmrc_income_tax_stats() returns expected structure", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_income_tax_stats()

  expect_s3_class(out, "hmrc_tbl")
  expect_named(out, c("tax_year", "income_range", "income_lower_gbp",
                      "taxpayers_thousands", "total_income_gbp_m",
                      "tax_liability_gbp_m", "average_rate_pct",
                      "average_tax_gbp"))
  expect_true(all(grepl("^\\d{4}-\\d{2}$", out$tax_year)))

  m <- hmrc_meta(out)
  expect_equal(m$dataset, "income_tax_liabilities_by_range")
  expect_equal(m$cell_methods, "liabilities")
})

test_that("hmrc_income_tax_stats() warns on unknown tax year", {
  skip_on_cran()
  skip_if_offline()

  expect_warning(
    out <- hmrc_income_tax_stats(tax_year = "1900-01"),
    "Tax year"
  )
  expect_equal(nrow(out), 0L)
})

test_that("hmrc_tax_gap() returns expected structure", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_tax_gap()

  expect_s3_class(out, "hmrc_tbl")
  expect_named(out, c("tax_year", "tax", "taxpayer_type", "component",
                      "gap_pct", "gap_gbp_bn", "uncertainty"))

  m <- hmrc_meta(out)
  expect_equal(m$dataset, "tax_gap_annual")
})

test_that("hmrc_tax_gap() filters by tax type", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_tax_gap(tax = "VAT")
  expect_true(all(out$tax == "VAT") || nrow(out) == 0L)
})

test_that("new_hmrc_tbl() attaches expected metadata", {
  df <- data.frame(date = as.Date("2024-01-01"), value = 100)
  out <- hmrc:::new_hmrc_tbl(
    df,
    dataset          = "demo",
    hmrc_publication = "Demo dataset",
    source_url       = "https://example.com/page",
    attachment_url   = "https://example.com/file.ods",
    slug             = "demo-slug",
    cell_methods     = "cash",
    frequency        = "monthly"
  )

  expect_s3_class(out, "hmrc_tbl")
  expect_s3_class(out, "data.frame")

  m <- hmrc_meta(out)
  expect_type(m, "list")
  expect_equal(m$dataset, "demo")
  expect_equal(m$hmrc_publication, "Demo dataset")
  expect_equal(m$source_url, "https://example.com/page")
  expect_equal(m$attachment_url, "https://example.com/file.ods")
  expect_equal(m$cell_methods, "cash")
  expect_equal(m$frequency, "monthly")
  expect_true(inherits(m$fetched_at, "POSIXt"))
  expect_true(is.na(m$vintage_date))
})

test_that("print.hmrc_tbl() emits a provenance header", {
  df  <- data.frame(date = as.Date("2024-01-01"), value = 100)
  out <- hmrc:::new_hmrc_tbl(df, hmrc_publication = "Header test",
                             source_url = "https://example.com/p")

  txt <- capture.output(print(out))
  expect_true(any(grepl("Header test", txt)))
  expect_true(any(grepl("Source: https://example.com/p", txt)))
  expect_true(any(grepl("Vintage: latest", txt)))
})

test_that("as.data.frame.hmrc_tbl() strips class and metadata", {
  df  <- data.frame(date = as.Date("2024-01-01"), value = 100)
  out <- hmrc:::new_hmrc_tbl(df, hmrc_publication = "Strip test")

  bare <- as.data.frame(out)
  expect_false(inherits(bare, "hmrc_tbl"))
  expect_null(attr(bare, "hmrc_meta"))
  expect_s3_class(bare, "data.frame")
})

test_that("subsetting [.hmrc_tbl preserves provenance", {
  df  <- data.frame(date = as.Date(c("2024-01-01", "2024-02-01")), value = c(100, 200))
  out <- hmrc:::new_hmrc_tbl(df, hmrc_publication = "Subset test")

  sub <- out[1, ]
  expect_s3_class(sub, "hmrc_tbl")
  expect_equal(hmrc_meta(sub)$hmrc_publication, "Subset test")
})

test_that("hmrc_meta() returns NULL for non-hmrc_tbl objects", {
  expect_null(hmrc_meta(data.frame(x = 1)))
  expect_null(hmrc_meta(NULL))
})

test_that("hmrc_vat() rejects unknown measures", {
  expect_error(
    hmrc_vat(measure = "not_a_measure"),
    "Unknown measure"
  )
})

test_that("hmrc_vat() returns expected structure with provenance", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_vat(measure = "total", start = "2020-01", end = "2020-12")

  expect_s3_class(out, "hmrc_tbl")
  expect_named(out, c("date", "measure", "description", "receipts_gbp_m"))
  expect_s3_class(out$date, "Date")
  expect_true(all(out$measure == "total"))

  m <- hmrc_meta(out)
  expect_equal(m$dataset, "vat_monthly")
  expect_equal(m$frequency, "monthly")
})

test_that("hmrc_vat() respects date filters", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_vat(measure = "total", start = "2020-01", end = "2020-06")
  expect_true(all(out$date >= as.Date("2020-01-01")))
  expect_true(all(out$date <= as.Date("2020-06-30")))
})

test_that("hmrc_capital_gains() rejects unknown measure", {
  expect_error(
    hmrc_capital_gains(measure = "not_a_measure"),
    "Unknown measure"
  )
})

test_that("hmrc_capital_gains() returns expected structure", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_capital_gains(measure = "tax_total_gbp_m")

  expect_s3_class(out, "hmrc_tbl")
  expect_named(out, c("tax_year", "measure", "value"))
  expect_true(all(out$measure == "tax_total_gbp_m"))
  expect_true(all(grepl("^\\d{4}-\\d{2}$", out$tax_year)))
  expect_type(out$value, "double")

  m <- hmrc_meta(out)
  expect_equal(m$dataset, "capital_gains_tax_annual")
  expect_equal(m$frequency, "annual")
})

test_that("hmrc_capital_gains() filters by year", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_capital_gains(tax_year = "2020-21")
  expect_true(all(out$tax_year == "2020-21") || nrow(out) == 0L)
})

test_that("hmrc_capital_gains() warns on unknown year", {
  skip_on_cran()
  skip_if_offline()

  expect_warning(
    out <- hmrc_capital_gains(tax_year = "1900-01"),
    "Tax year"
  )
  expect_equal(nrow(out), 0L)
})

test_that("hmrc_tobacco_duties() rejects unknown product", {
  expect_error(
    hmrc_tobacco_duties(product = "snus"),
    "Unknown product"
  )
})

test_that("hmrc_tobacco_duties() returns expected structure", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_tobacco_duties(product = "cigarettes",
                             start = "2020-01", end = "2020-12")

  expect_s3_class(out, "hmrc_tbl")
  expect_named(out, c("date", "product", "description", "receipts_gbp_m"))
  expect_true(all(out$product == "cigarettes"))

  m <- hmrc_meta(out)
  expect_equal(m$dataset, "tobacco_duties_monthly")
})

test_that("hmrc_creative_industries() rejects unknown sector", {
  expect_error(
    hmrc_creative_industries(sector = "podcast"),
    "Unknown sector"
  )
})

test_that("hmrc_creative_industries() returns expected structure", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_creative_industries(sector = "film")

  expect_s3_class(out, "hmrc_tbl")
  expect_named(out, c("sector", "tax_year", "companies", "claims",
                      "productions", "relief_gbp_m", "status"))
  expect_true(all(out$sector == "film"))
  expect_true(all(grepl("^\\d{4}-\\d{2}$", out$tax_year)))

  m <- hmrc_meta(out)
  expect_equal(m$dataset, "creative_industries_reliefs")
})

test_that("hmrc_creative_industries() covers all eight sectors", {
  skip_on_cran()
  skip_if_offline()

  out <- hmrc_creative_industries()
  expect_setequal(
    unique(out$sector),
    c("film", "high_end_tv", "animation", "childrens_tv",
      "video_games", "theatre", "orchestra", "museum")
  )
})

#' Download HMRC tobacco duty receipts
#'
#' Downloads and tidies the HMRC Tobacco Bulletin, which reports monthly
#' tobacco products duty receipts by product type. Data runs from January
#' 1991 to the most recent published month, updated twice per year
#' (February and August).
#'
#' @param product Character vector or `NULL` (default = all products).
#'   Valid values: `"cigarettes"`, `"cigars"`, `"hand_rolling"`,
#'   `"other"`, `"total"`.
#' @param start Character `"YYYY-MM"` or a `Date` object.
#' @param end Character `"YYYY-MM"` or a `Date` object.
#' @param cache Logical. Use cached file if available (default `TRUE`).
#'
#' @return An `hmrc_tbl` with columns `date`, `product`, `description`,
#'   `receipts_gbp_m`.
#'
#' @source
#' <https://www.gov.uk/government/statistics/tobacco-bulletin>
#'
#' @examples
#' \donttest{
#' op <- options(hmrc.cache_dir = tempdir())
#' hmrc_tobacco_duties(start = "2015-01")
#' hmrc_tobacco_duties(product = "cigarettes")
#' options(op)
#' }
#'
#' @family data fetchers
#' @export
hmrc_tobacco_duties <- function(product = NULL,
                                start   = NULL,
                                end     = NULL,
                                cache   = TRUE) {

  valid_products <- c("cigarettes", "cigars", "hand_rolling", "other", "total")
  if (!is.null(product)) {
    bad <- setdiff(product, valid_products)
    if (length(bad) > 0) {
      cli::cli_abort(c(
        "Unknown product{?s}: {.val {bad}}",
        "i" = "Valid values: {.val {valid_products}}"
      ))
    }
  }
  if (!is.null(start)) parse_month_arg(start, "start")
  if (!is.null(end))   parse_month_arg(end,   "end")

  slug <- "tobacco-bulletin"
  loc  <- resolve_govuk_attachment(slug, "Tobacco_Tab")
  path <- download_cached(loc$attachment_url, cache = cache)

  cli::cli_progress_step("Parsing data")
  out <- parse_tobacco_table(path)

  if (!is.null(product)) out <- out[out$product %in% product, ]
  out <- filter_dates(out, start, end)
  rownames(out) <- NULL

  cli::cli_progress_done()

  new_hmrc_tbl(
    out,
    dataset          = "tobacco_duties_monthly",
    hmrc_publication = "Tobacco Bulletin",
    source_url       = loc$page_url,
    attachment_url   = loc$attachment_url,
    slug             = slug,
    cell_methods     = "cash",
    frequency        = "monthly"
  )
}

# ── Internal parser ────────────────────────────────────────────────────────────

TOBACCO_COLS <- c(
  "cigarettes"   = "Cigarettes",
  "cigars"       = "Cigars",
  "hand_rolling" = "Hand-rolling tobacco",
  "other"        = "Other tobacco products",
  "total"        = "Total tobacco products"
)

parse_tobacco_table <- function(path) {
  raw <- readODS::read_ods(path, sheet = "Table_1_receipts",
                           col_names = FALSE, as_tibble = FALSE)

  monthly_start <- NA_integer_
  for (i in seq_len(nrow(raw))) {
    cell <- trimws(as.character(raw[i, 1]))
    if (grepl("Table.*1c|by month", cell, ignore.case = TRUE)) {
      monthly_start <- i + 1
      break
    }
  }
  if (is.na(monthly_start)) {
    for (i in seq_len(nrow(raw))) {
      cell <- trimws(as.character(raw[i, 1]))
      if (grepl("^[A-Za-z]+ \\d{4}$", cell)) {
        monthly_start <- i
        break
      }
    }
  }
  if (is.na(monthly_start)) {
    cli::cli_abort("Could not locate monthly data in tobacco bulletin. Please file an issue.")
  }

  data_rows <- raw[monthly_start:nrow(raw), ]

  dates_raw <- trimws(as.character(data_rows[[1]]))
  dates     <- suppressWarnings(
    as.Date(paste0("01 ", dates_raw), format = "%d %B %Y")
  )
  valid     <- !is.na(dates)
  dates     <- dates[valid]
  data_rows <- data_rows[valid, ]

  col_keys  <- names(TOBACCO_COLS)
  col_descs <- unname(TOBACCO_COLS)

  results <- list()
  for (j in seq_along(col_keys)) {
    col_idx <- j + 1
    if (col_idx > ncol(data_rows)) break
    vals <- suppressWarnings(as.numeric(as.character(data_rows[[col_idx]])))
    results[[j]] <- data.frame(
      date           = dates,
      product        = col_keys[j],
      description    = col_descs[j],
      receipts_gbp_m = vals,
      stringsAsFactors = FALSE
    )
  }

  out <- do.call(rbind, results)
  out <- out[order(out$product, out$date), ]
  rownames(out) <- NULL
  out
}

#' Clear the local HMRC data cache
#'
#' Deletes locally cached data files downloaded by the package. By default
#' all cached files are removed; pass `max_age_days` to remove only files
#' older than that.
#'
#' @param max_age_days Numeric or `NULL`. If `NULL` (default), every cached
#'   file is removed. If a number, only files modified more than that many
#'   days ago are removed.
#'
#' @return Invisibly returns the number of files deleted.
#'
#' @examples
#' \donttest{
#' op <- options(hmrc.cache_dir = tempdir())
#' hmrc_clear_cache()
#' hmrc_clear_cache(max_age_days = 30)
#' options(op)
#' }
#'
#' @family infrastructure
#' @export
hmrc_clear_cache <- function(max_age_days = NULL) {
  if (!is.null(max_age_days)) {
    if (!is.numeric(max_age_days) || length(max_age_days) != 1L || max_age_days < 0) {
      cli::cli_abort("{.arg max_age_days} must be a non-negative number.")
    }
  }

  cache_dir <- hmrc_cache_dir()

  if (!dir.exists(cache_dir)) {
    cli::cli_inform("No cache directory found \u2014 nothing to clear.")
    return(invisible(0L))
  }

  files <- list.files(cache_dir, full.names = TRUE)

  if (length(files) == 0) {
    cli::cli_inform("Cache is already empty.")
    return(invisible(0L))
  }

  if (!is.null(max_age_days)) {
    mtime    <- file.mtime(files)
    age_days <- as.numeric(difftime(Sys.time(), mtime, units = "days"))
    files    <- files[age_days > max_age_days]
  }

  if (length(files) == 0) {
    cli::cli_inform("No files older than {max_age_days} day{?s} found.")
    return(invisible(0L))
  }

  unlink(files)
  cli::cli_inform("Deleted {length(files)} cached file{?s}.")
  invisible(length(files))
}

test_that("hmrc_clear_cache() works when cache is empty or absent", {
  tmp <- file.path(tempdir(), paste0("hmrc-clear-cache-", Sys.getpid()))
  unlink(tmp, recursive = TRUE)
  op <- options(hmrc.cache_dir = tmp)
  on.exit(options(op), add = TRUE)

  expect_no_error(hmrc_clear_cache())
})

test_that("hmrc_clear_cache() accepts valid max_age_days", {
  expect_no_error(hmrc_clear_cache(max_age_days = 30))
  expect_no_error(hmrc_clear_cache(max_age_days = 0))
})

test_that("hmrc_clear_cache() rejects invalid max_age_days", {
  expect_error(hmrc_clear_cache(max_age_days = -1))
  expect_error(hmrc_clear_cache(max_age_days = "30"))
})

test_that("hmrc_clear_cache() returns an invisible integer", {
  result <- hmrc_clear_cache()
  expect_type(result, "integer")
})

test_that("hmrc_clear_cache() removes files older than max_age_days", {
  tmp <- file.path(tempdir(), paste0("hmrc-clear-cache-age-", Sys.getpid()))
  dir.create(tmp, recursive = TRUE, showWarnings = FALSE)
  op <- options(hmrc.cache_dir = tmp)
  on.exit({
    options(op)
    unlink(tmp, recursive = TRUE)
  }, add = TRUE)

  old <- file.path(tmp, "old.ods")
  new <- file.path(tmp, "new.ods")
  writeLines("a", old)
  writeLines("a", new)
  Sys.setFileTime(old, Sys.time() - 60 * 60 * 24 * 90)

  removed <- hmrc_clear_cache(max_age_days = 30)
  expect_equal(removed, 1L)
  expect_false(file.exists(old))
  expect_true(file.exists(new))
})

#' Download HMRC VAT receipts
#'
#' Downloads and tidies the HMRC VAT Annual Statistics bulletin, which
#' reports monthly VAT receipts broken down into payments, repayments,
#' import VAT, and home VAT. Monthly data runs from April 1973; the
#' bulletin is published annually (December).
#'
#' @param measure Character vector or `NULL` (default = all measures).
#'   Valid values: `"total"`, `"payments"`, `"repayments"`,
#'   `"import_vat"`, `"home_vat"`.
#' @param start Character `"YYYY-MM"` or a `Date` object.
#' @param end Character `"YYYY-MM"` or a `Date` object.
#' @param cache Logical. Use cached file if available (default `TRUE`).
#'
#' @return An `hmrc_tbl` with columns `date`, `measure`, `description`,
#'   `receipts_gbp_m`. Repayments are negative.
#'
#' @details
#' Note that early years (pre-1985) have suppressed payment and repayment
#' splits; only the total is available for those periods. From January 2021,
#' import VAT collected via postponed VAT accounting is recorded within
#' payments and repayments rather than the import VAT column.
#'
#' @source
#' <https://www.gov.uk/government/statistics/value-added-tax-vat-annual-statistics>
#'
#' @examples
#' \donttest{
#' op <- options(hmrc.cache_dir = tempdir())
#' hmrc_vat(measure = "total", start = "2010-01")
#' hmrc_vat(start = "2020-01")
#' options(op)
#' }
#'
#' @family data fetchers
#' @export
hmrc_vat <- function(measure = NULL,
                     start   = NULL,
                     end     = NULL,
                     cache   = TRUE) {

  valid_measures <- c("total", "payments", "repayments", "import_vat", "home_vat")
  if (!is.null(measure)) {
    bad <- setdiff(measure, valid_measures)
    if (length(bad) > 0) {
      cli::cli_abort(c(
        "Unknown measure{?s}: {.val {bad}}",
        "i" = "Valid values: {.val {valid_measures}}"
      ))
    }
  }
  if (!is.null(start)) parse_month_arg(start, "start")
  if (!is.null(end))   parse_month_arg(end,   "end")

  slug <- "value-added-tax-vat-annual-statistics"
  loc  <- resolve_govuk_attachment(slug, "Annual_UK_VAT")
  path <- download_cached(loc$attachment_url, cache = cache)

  cli::cli_progress_step("Parsing data")
  out <- parse_vat_table(path)

  if (!is.null(measure)) out <- out[out$measure %in% measure, ]
  out <- filter_dates(out, start, end)
  rownames(out) <- NULL

  cli::cli_progress_done()

  new_hmrc_tbl(
    out,
    dataset          = "vat_monthly",
    hmrc_publication = "VAT annual statistics (monthly receipts)",
    source_url       = loc$page_url,
    attachment_url   = loc$attachment_url,
    slug             = slug,
    cell_methods     = "cash",
    frequency        = "monthly"
  )
}

# ── Internal parser ────────────────────────────────────────────────────────────

VAT_COLS <- c(
  "payments"   = "VAT payments",
  "repayments" = "VAT repayments",
  "import_vat" = "Import VAT",
  "home_vat"   = "Home VAT",
  "total"      = "Total VAT receipts"
)

parse_vat_table <- function(path) {
  raw <- readODS::read_ods(path, sheet = "T1_VAT_receipts",
                           col_names = FALSE, as_tibble = FALSE)

  monthly_start <- NA_integer_
  for (i in seq_len(nrow(raw))) {
    cell <- trimws(as.character(raw[i, 1]))
    if (grepl("Table.*1b|by calendar year and month", cell, ignore.case = TRUE)) {
      monthly_start <- i + 1
      break
    }
  }
  if (is.na(monthly_start)) {
    cli::cli_abort("Could not locate monthly VAT data. Please file an issue.")
  }

  data_rows <- raw[monthly_start:nrow(raw), ]

  dates_raw <- trimws(as.character(data_rows[[1]]))
  dates     <- suppressWarnings(
    as.Date(paste0("01 ", dates_raw), format = "%d %B %Y")
  )
  valid     <- !is.na(dates)
  dates     <- dates[valid]
  data_rows <- data_rows[valid, ]

  col_keys  <- names(VAT_COLS)
  col_descs <- unname(VAT_COLS)

  results <- list()
  for (j in seq_along(col_keys)) {
    col_idx <- j + 1
    if (col_idx > ncol(data_rows)) break
    raw_vals <- as.character(data_rows[[col_idx]])
    raw_vals[grepl("\\[", raw_vals)] <- NA_character_
    vals <- suppressWarnings(as.numeric(raw_vals))

    results[[j]] <- data.frame(
      date           = dates,
      measure        = col_keys[j],
      description    = col_descs[j],
      receipts_gbp_m = vals,
      stringsAsFactors = FALSE
    )
  }

  out <- do.call(rbind, results)
  out <- out[order(out$measure, out$date), ]
  rownames(out) <- NULL
  out
}

#' Download HMRC Income Tax liabilities by income range
#'
#' Downloads and tidies HMRC Table 2.5, which reports the number of Income Tax
#' payers and their liabilities grouped by total income range, for each
#' available tax year. Numbers of taxpayers are in thousands; amounts are in
#' millions of pounds unless otherwise noted. Published annually in May/June.
#'
#' @param tax_year Character vector or `NULL` (default = all years). Filter to
#'   specific tax years, e.g. `"2023-24"`.
#' @param cache Logical. Use cached file if available (default `TRUE`).
#'
#' @return An `hmrc_tbl` with columns `tax_year`, `income_range`,
#'   `income_lower_gbp`, `taxpayers_thousands`, `total_income_gbp_m`,
#'   `tax_liability_gbp_m`, `average_rate_pct`, `average_tax_gbp`.
#'
#' @details
#' The earliest tax year with outturn data is based on the Survey of Personal
#' Incomes; later years are projected estimates. Values suppressed for small
#' sample sizes are returned as `NA`.
#'
#' @source
#' <https://www.gov.uk/government/statistics/income-tax-liabilities-by-income-range>
#'
#' @examples
#' \donttest{
#' op <- options(hmrc.cache_dir = tempdir())
#' hmrc_income_tax_stats()
#' hmrc_income_tax_stats(tax_year = "2023-24")
#' options(op)
#' }
#'
#' @family data fetchers
#' @export
hmrc_income_tax_stats <- function(tax_year = NULL,
                                  cache    = TRUE) {

  slug <- "income-tax-liabilities-by-income-range"
  loc  <- resolve_govuk_attachment(slug, "Table_2\\.5")
  path <- download_cached(loc$attachment_url, cache = cache)

  cli::cli_progress_step("Parsing data")

  sheets      <- readODS::list_ods_sheets(path)
  data_sheets <- sheets[grepl("^2_5_IT_Liabilities_", sheets)]

  if (length(data_sheets) == 0) {
    cli::cli_abort("No data sheets found in Table 2.5 file. Please file an issue.")
  }

  results <- lapply(data_sheets, function(s) parse_income_tax_sheet(path, s))
  out <- do.call(rbind, results)

  if (!is.null(tax_year)) {
    bad <- setdiff(tax_year, unique(out$tax_year))
    if (length(bad) > 0) {
      cli::cli_warn(c(
        "Tax year{?s} not found: {.val {bad}}",
        "i" = "Available years: {.val {unique(out$tax_year)}}"
      ))
    }
    out <- out[out$tax_year %in% tax_year, ]
  }
  rownames(out) <- NULL

  cli::cli_progress_done()

  new_hmrc_tbl(
    out,
    dataset          = "income_tax_liabilities_by_range",
    hmrc_publication = "Income Tax liabilities by income range (Table 2.5)",
    source_url       = loc$page_url,
    attachment_url   = loc$attachment_url,
    slug             = slug,
    cell_methods     = "liabilities",
    frequency        = "annual"
  )
}

# -- Internal parser -----------------------------------------------------------

parse_income_tax_sheet <- function(path, sheet) {
  yr <- sub("^2_5_IT_Liabilities_", "", sheet)

  raw <- readODS::read_ods(path, sheet = sheet,
                           col_names = FALSE, as_tibble = FALSE)

  data_rows <- raw[4:nrow(raw), ]
  range_raw <- trimws(as.character(data_rows[[1]]))
  valid     <- nzchar(range_raw) &
    !grepl("End of|^NA$", range_raw, ignore.case = TRUE)
  data_rows <- data_rows[valid, ]
  range_raw <- range_raw[valid]

  if (nrow(data_rows) == 0) {
    cli::cli_abort("No data rows found in sheet {.val {sheet}}.")
  }

  income_lower <- vapply(range_raw, function(x) {
    if (grepl("All Ranges", x, ignore.case = TRUE)) return(NA_real_)
    cleaned <- gsub("[\u00a3,+]", "", trimws(x))
    suppressWarnings(as.numeric(cleaned))
  }, numeric(1), USE.NAMES = FALSE)

  income_range <- vapply(range_raw, function(x) {
    if (grepl("All Ranges", x, ignore.case = TRUE)) return("All Ranges")
    gsub("[\u00a3,]", "", trimws(x))
  }, character(1), USE.NAMES = FALSE)

  parse_num <- function(col) {
    vals <- trimws(as.character(data_rows[[col]]))
    vals <- gsub("[%\u00a3,]", "", vals)
    vals[grepl("no estimate", vals, ignore.case = TRUE)] <- NA_character_
    suppressWarnings(as.numeric(vals))
  }

  data.frame(
    tax_year            = yr,
    income_range        = income_range,
    income_lower_gbp    = income_lower,
    taxpayers_thousands = parse_num(10),
    total_income_gbp_m  = parse_num(11),
    tax_liability_gbp_m = parse_num(12),
    average_rate_pct    = parse_num(13),
    average_tax_gbp     = parse_num(14),
    stringsAsFactors    = FALSE
  )
}

#' Download HMRC tax gap estimates
#'
#' Downloads and tidies the HMRC Measuring the Tax Gap publication, which
#' estimates the difference between the tax theoretically owed and the tax
#' actually collected, broken down by tax type and taxpayer group. Published
#' annually in June, covering the most recent financial year.
#'
#' @param tax Character vector or `NULL` (default = all taxes). Filter by
#'   tax type, e.g. `"Income Tax"`, `"VAT"`, `"Corporation Tax"`. Use
#'   `unique(hmrc_tax_gap()$tax)` to see all available values.
#' @param cache Logical. Use cached file if available (default `TRUE`).
#'
#' @return An `hmrc_tbl` with columns `tax_year`, `tax`, `taxpayer_type`,
#'   `component`, `gap_pct`, `gap_gbp_bn`, `uncertainty`.
#'
#' @details
#' The tax gap publication is cross-sectional: each edition covers a single
#' financial year. This function returns data for the most recent edition
#' available on GOV.UK. Historical estimates back to 2005-06 are available
#' in a separate HMRC publication (Measuring the Tax Gap: Time Series).
#'
#' @source
#' <https://www.gov.uk/government/statistics/measuring-tax-gaps>
#'
#' @examples
#' \donttest{
#' op <- options(hmrc.cache_dir = tempdir())
#' hmrc_tax_gap()
#' hmrc_tax_gap(tax = "VAT")
#' options(op)
#' }
#'
#' @family data fetchers
#' @export
hmrc_tax_gap <- function(tax   = NULL,
                         cache = TRUE) {

  slug <- "measuring-tax-gaps-tables"
  loc  <- resolve_govuk_attachment(slug, "Measuring_tax_gap_online")
  path <- download_cached(loc$attachment_url, cache = cache)

  cli::cli_progress_step("Parsing data")
  out <- parse_tax_gap_table(path)

  if (!is.null(tax)) {
    bad <- setdiff(tax, unique(out$tax))
    if (length(bad) > 0) {
      cli::cli_warn(c(
        "Tax type{?s} not found: {.val {bad}}",
        "i" = "Call {.code hmrc_tax_gap()} with no arguments to see available values."
      ))
    }
    out <- out[out$tax %in% tax, ]
  }
  rownames(out) <- NULL

  cli::cli_progress_done()

  new_hmrc_tbl(
    out,
    dataset          = "tax_gap_annual",
    hmrc_publication = "Measuring Tax Gaps",
    source_url       = loc$page_url,
    attachment_url   = loc$attachment_url,
    slug             = slug,
    cell_methods     = "estimates",
    frequency        = "annual"
  )
}

# -- Internal parser -----------------------------------------------------------

parse_tax_gap_table <- function(path) {
  raw <- readxl::read_excel(path, sheet = "Table 1.1",
                            col_names = FALSE, .name_repair = "minimal")
  raw <- as.data.frame(raw, stringsAsFactors = FALSE)

  tax_year <- NA_character_
  for (i in seq_len(min(6L, nrow(raw)))) {
    cell <- trimws(as.character(raw[i, 1L]))
    m    <- regmatches(cell, regexpr("20\\d{2}[[:space:]]to[[:space:]]20\\d{2}", cell))
    if (length(m) == 1L && nzchar(m)) {
      yrs <- regmatches(m, gregexpr("20\\d{2}", m))[[1]]
      tax_year <- paste0(yrs[1], "-", substr(yrs[2], 3, 4))
      break
    }
    m2 <- regmatches(cell, regexpr("20\\d{2}-\\d{2}", cell))
    if (length(m2) == 1L && nzchar(m2)) { tax_year <- m2; break }
  }

  data_rows <- raw[7:nrow(raw), ]

  valid <- !is.na(suppressWarnings(as.numeric(
    gsub("<|>|,", "", trimws(as.character(data_rows[[5]])))
  ))) | !is.na(suppressWarnings(as.numeric(
    gsub("%", "", trimws(as.character(data_rows[[4]])))
  )))
  data_rows <- data_rows[valid, ]

  if (nrow(data_rows) == 0) {
    cli::cli_abort("Could not locate data rows in tax gap table. Please file an issue.")
  }

  pct_raw    <- gsub("[%<>]", "", trimws(as.character(data_rows[[4]])))
  gap_pct    <- suppressWarnings(as.numeric(pct_raw))

  abs_raw    <- gsub("[\u00a3<>]", "", trimws(as.character(data_rows[[5]])))
  gap_gbp_bn <- suppressWarnings(as.numeric(abs_raw))

  out <- data.frame(
    tax_year      = tax_year,
    tax           = trimws(as.character(data_rows[[1]])),
    taxpayer_type = trimws(as.character(data_rows[[2]])),
    component     = trimws(as.character(data_rows[[3]])),
    gap_pct       = gap_pct,
    gap_gbp_bn    = gap_gbp_bn,
    uncertainty   = trimws(as.character(data_rows[[6]])),
    stringsAsFactors = FALSE
  )

  out <- out[order(out$tax, out$taxpayer_type, out$component), ]
  rownames(out) <- NULL
  out
}

#' Download HMRC Capital Gains Tax statistics
#'
#' Downloads and tidies HMRC Table 1 of the Capital Gains Tax statistics
#' bulletin: estimated number of CGT taxpayers, amounts of gains, and tax
#' liabilities by year of disposal. Data runs from 1987-88 onwards and is
#' published annually each summer.
#'
#' @param tax_year Character vector or `NULL` (default = all years).
#'   Filter to specific tax years, e.g. `"2022-23"`.
#' @param measure Character vector or `NULL` (default = all measures).
#'   Valid values: `"taxpayers_total_thousands"`,
#'   `"taxpayers_individuals_thousands"`, `"taxpayers_trusts_thousands"`,
#'   `"males_pct"`, `"females_pct"`,
#'   `"gains_individuals_gbp_m"`, `"gains_trusts_gbp_m"`,
#'   `"gains_total_gbp_m"`, `"tax_individuals_gbp_m"`,
#'   `"tax_trusts_gbp_m"`, `"tax_total_gbp_m"`.
#' @param cache Logical. Use cached file if available (default `TRUE`).
#'
#' @return An `hmrc_tbl` (long format) with columns `tax_year`, `measure`,
#'   `value`. Numbers of taxpayers are in thousands; amounts in millions
#'   of pounds; percentages in percent. Suppressed cells return `NA`.
#'
#' @details
#' Gains are after the deduction of taper relief and losses plus attributed
#' gains, but before deduction of the Annual Exempt Amount. Trusts include
#' personal representatives of the deceased. Statistics for the latest two
#' tax years are provisional and are revised in subsequent publications as
#' late returns arrive.
#'
#' @source
#' <https://www.gov.uk/government/organisations/hm-revenue-customs/series/capital-gains-tax-statistics>
#'
#' @examples
#' \donttest{
#' op <- options(hmrc.cache_dir = tempdir())
#' hmrc_capital_gains()
#' hmrc_capital_gains(measure = "tax_total_gbp_m")
#' hmrc_capital_gains(tax_year = "2022-23")
#' options(op)
#' }
#'
#' @family data fetchers
#' @export
hmrc_capital_gains <- function(tax_year = NULL,
                               measure  = NULL,
                               cache    = TRUE) {

  valid_measures <- names(CGT_T1_COLS)
  if (!is.null(measure)) {
    bad <- setdiff(measure, valid_measures)
    if (length(bad) > 0) {
      cli::cli_abort(c(
        "Unknown measure{?s}: {.val {bad}}",
        "i" = "Valid values: {.val {valid_measures}}"
      ))
    }
  }

  slug <- "capital-gains-tax-statistics"
  loc  <- resolve_govuk_attachment(slug, "Table_1_.*Taxpayer_numbers.*\\.ods")
  path <- download_cached(loc$attachment_url, cache = cache)

  cli::cli_progress_step("Parsing data")
  out <- parse_cgt_table1(path)

  if (!is.null(tax_year)) {
    bad <- setdiff(tax_year, unique(out$tax_year))
    if (length(bad) > 0) {
      cli::cli_warn(c(
        "Tax year{?s} not found: {.val {bad}}",
        "i" = "Available years: {.val {unique(out$tax_year)}}"
      ))
    }
    out <- out[out$tax_year %in% tax_year, ]
  }
  if (!is.null(measure)) out <- out[out$measure %in% measure, ]
  rownames(out) <- NULL

  cli::cli_progress_done()

  new_hmrc_tbl(
    out,
    dataset          = "capital_gains_tax_annual",
    hmrc_publication = "Capital Gains Tax statistics (Table 1)",
    source_url       = loc$page_url,
    attachment_url   = loc$attachment_url,
    slug             = slug,
    cell_methods     = "liabilities",
    frequency        = "annual"
  )
}

# ── Internal parser ────────────────────────────────────────────────────────────
#
# CGT Table 1 layout (HMRC publication, July release):
#   Row 8 = column headers
#   Row 9+ = data, one row per tax year (e.g. "1987 to 1988")
#   Col 1: Year of disposal
#   Cols 2-12: 11 measures (taxpayers, gains, tax for individuals / trusts /
#              total; plus males/females percentages)
# Suppressed cells hold "[Unavailable]" or similar text and become NA.

CGT_T1_COLS <- c(
  taxpayers_individuals_thousands = "Number of individuals",
  males_pct                       = "Males as %",
  females_pct                     = "Females as %",
  gains_individuals_gbp_m         = "Amounts of gains for individuals",
  tax_individuals_gbp_m           = "Amounts of tax for individuals",
  taxpayers_trusts_thousands      = "Number of trusts",
  gains_trusts_gbp_m              = "Amounts of gains for trusts",
  tax_trusts_gbp_m                = "Amounts of tax for trusts",
  taxpayers_total_thousands       = "Total taxpayers",
  gains_total_gbp_m               = "Total amounts of gains",
  tax_total_gbp_m                 = "Total amounts of tax"
)

parse_cgt_table1 <- function(path) {
  raw <- readODS::read_ods(path, sheet = "Table_1",
                           col_names = FALSE, as_tibble = FALSE)

  # Find the row with "Year of disposal" anchor (header row)
  hdr_row <- NA_integer_
  for (i in seq_len(nrow(raw))) {
    cell <- trimws(as.character(raw[i, 1]))
    if (identical(tolower(cell), "year of disposal")) {
      hdr_row <- i
      break
    }
  }
  if (is.na(hdr_row)) {
    cli::cli_abort("Could not locate header row in CGT Table 1. Please file an issue.")
  }

  data_rows <- raw[(hdr_row + 1L):nrow(raw), ]

  # Tax year is in col 1: "1987 to 1988" -> "1987-88"
  yr_raw <- trimws(as.character(data_rows[[1]]))
  # Strip "[Note ...]" footnote markers
  yr_raw <- trimws(gsub("\\[Note[^\\]]*\\]", "", yr_raw))
  is_year <- grepl("^\\d{4}\\s+to\\s+\\d{4}$", yr_raw)
  data_rows <- data_rows[is_year, ]
  yr_raw    <- yr_raw[is_year]

  to_yyyy_yy <- function(x) {
    parts <- regmatches(x, gregexpr("\\d{4}", x))
    vapply(parts, function(p) {
      if (length(p) >= 2L) paste0(p[1], "-", substr(p[2], 3, 4)) else NA_character_
    }, character(1))
  }
  tax_years <- to_yyyy_yy(yr_raw)

  parse_num <- function(x) {
    x <- gsub(",", "", trimws(as.character(x)))
    x[grepl("\\[", x)] <- NA_character_
    suppressWarnings(as.numeric(x))
  }

  results <- list()
  measure_keys <- names(CGT_T1_COLS)
  for (j in seq_along(measure_keys)) {
    col_idx <- j + 1L
    if (col_idx > ncol(data_rows)) break
    results[[j]] <- data.frame(
      tax_year = tax_years,
      measure  = measure_keys[j],
      value    = parse_num(data_rows[[col_idx]]),
      stringsAsFactors = FALSE
    )
  }

  out <- do.call(rbind, results)
  out <- out[order(out$measure, out$tax_year), ]
  rownames(out) <- NULL
  out
}

#' Download HMRC Creative Industries tax relief statistics
#'
#' Downloads and tidies Table 1 (annual time series) of the HMRC Creative
#' Industries Statistics for each of the eight creative-industries reliefs:
#' Film, High-end TV, Animation, Children's TV, Video Games, Theatre,
#' Orchestra, and Museums and Galleries Exhibition. Published annually
#' each August.
#'
#' @param sector Character vector or `NULL` (default = all sectors).
#'   Valid values: `"film"`, `"high_end_tv"`, `"animation"`,
#'   `"childrens_tv"`, `"video_games"`, `"theatre"`, `"orchestra"`,
#'   `"museum"`.
#' @param tax_year Character vector or `NULL` (default = all years).
#' @param cache Logical. Use cached file if available (default `TRUE`).
#'
#' @return An `hmrc_tbl` with columns:
#'   \describe{
#'     \item{sector}{Character. Creative-industries sector identifier.}
#'     \item{tax_year}{Character. Tax year, e.g. `"2023-24"`.}
#'     \item{companies}{Numeric. Number of companies (where reported).}
#'     \item{claims}{Numeric. Number of claims (where reported).}
#'     \item{productions}{Numeric. Number of productions (films, games,
#'       etc., where reported).}
#'     \item{relief_gbp_m}{Numeric. Amount of relief paid, GBP million.}
#'     \item{status}{Character. HMRC revision status (e.g. `"Unchanged"`,
#'       `"Provisional"`).}
#'   }
#'
#' @details
#' Creative Industries reliefs are paid on an accruals basis. The latest
#' tax year in each sector's table is provisional and uplifted by HMRC for
#' claims not yet received; status is recorded in the `status` column.
#' Sector tables differ slightly in their column set (films track
#' `productions`, video games track `productions`, theatre tracks
#' `claims`/`productions`, museums track exhibitions, etc.); columns absent
#' from a sector's table are returned as `NA`.
#'
#' @source
#' <https://www.gov.uk/government/collections/creative-industries-statistics>
#'
#' @examples
#' \donttest{
#' op <- options(hmrc.cache_dir = tempdir())
#' hmrc_creative_industries(sector = "film")
#' hmrc_creative_industries(tax_year = "2023-24")
#' options(op)
#' }
#'
#' @family data fetchers
#' @export
hmrc_creative_industries <- function(sector   = NULL,
                                     tax_year = NULL,
                                     cache    = TRUE) {

  valid_sectors <- names(CREATIVE_SECTOR_SHEETS)
  if (!is.null(sector)) {
    bad <- setdiff(sector, valid_sectors)
    if (length(bad) > 0) {
      cli::cli_abort(c(
        "Unknown sector{?s}: {.val {bad}}",
        "i" = "Valid values: {.val {valid_sectors}}"
      ))
    }
  }

  # The publication slug includes the month: try latest-year August + September
  current_year <- as.integer(format(Sys.Date(), "%Y"))
  loc <- NULL
  candidates <- character(0)
  for (yr in seq(current_year, current_year - 2L)) {
    candidates <- c(candidates,
                    paste0("creative-industries-statistics-august-", yr),
                    paste0("creative-industries-statistics-september-", yr),
                    paste0("creative-industries-statistics-", yr))
  }
  for (slug in candidates) {
    loc <- tryCatch(
      resolve_govuk_attachment(slug, "Creative_industries_tables"),
      error = function(e) NULL
    )
    if (!is.null(loc)) break
  }
  if (is.null(loc)) {
    cli::cli_abort(c(
      "Could not find Creative Industries statistics on GOV.UK.",
      "i" = "Tried slugs spanning {current_year - 2L} to {current_year}.",
      "i" = "Check {.url https://www.gov.uk/government/collections/creative-industries-statistics}"
    ))
  }
  path <- download_cached(loc$attachment_url, cache = cache)

  cli::cli_progress_step("Parsing data")
  out <- parse_creative_industries(path)

  if (!is.null(sector))   out <- out[out$sector   %in% sector,   ]
  if (!is.null(tax_year)) out <- out[out$tax_year %in% tax_year, ]
  rownames(out) <- NULL

  cli::cli_progress_done()

  new_hmrc_tbl(
    out,
    dataset          = "creative_industries_reliefs",
    hmrc_publication = "Creative Industries tax relief statistics (Table 1)",
    source_url       = loc$page_url,
    attachment_url   = loc$attachment_url,
    slug             = loc$slug,
    cell_methods     = "counts/cash",
    frequency        = "annual"
  )
}

# ── Internal parser ────────────────────────────────────────────────────────────

CREATIVE_SECTOR_SHEETS <- c(
  film         = "Table_1_Film",
  high_end_tv  = "Table_1_High-end_TV",
  animation    = "Table_1_Animation",
  childrens_tv = "Table_1_Childrens_TV",
  video_games  = "Table_1_Video_games",
  theatre      = "Table_1_Theatre",
  orchestra    = "Table_1_Orchestra",
  museum       = "Table_1_Museum"
)

CREATIVE_HEADER_PATTERNS <- list(
  companies   = "number of companies",
  claims      = "number of claims",
  productions = "number of (films|games|productions|exhibitions|tours|orchestral concerts|theatrical productions)",
  relief_gbp_m= "amount of relief",
  status      = "^status$"
)

parse_creative_industries <- function(path) {
  available <- readODS::list_ods_sheets(path)

  results <- list()
  for (sec in names(CREATIVE_SECTOR_SHEETS)) {
    sheet <- CREATIVE_SECTOR_SHEETS[[sec]]
    if (!sheet %in% available) next

    raw <- readODS::read_ods(path, sheet = sheet,
                             col_names = FALSE, as_tibble = FALSE)

    # Header row: col 1 == "Year"
    hdr_row <- NA_integer_
    for (i in seq_len(nrow(raw))) {
      cell <- trimws(as.character(raw[i, 1]))
      if (identical(tolower(cell), "year")) {
        hdr_row <- i
        break
      }
    }
    if (is.na(hdr_row)) next

    header_row <- tolower(trimws(as.character(unlist(raw[hdr_row, ]))))
    data_rows  <- raw[(hdr_row + 1L):nrow(raw), ]

    yr_raw    <- trimws(as.character(data_rows[[1]]))
    is_year   <- grepl("^\\d{4}-\\d{2}$", yr_raw)
    data_rows <- data_rows[is_year, ]
    yr_raw    <- yr_raw[is_year]
    if (length(yr_raw) == 0L) next

    parse_num <- function(x) {
      x <- gsub(",", "", trimws(as.character(x)))
      x[grepl("\\[", x)] <- NA_character_
      suppressWarnings(as.numeric(x))
    }

    pick_col <- function(pattern) {
      idx <- which(grepl(pattern, header_row, perl = TRUE))
      if (length(idx) == 0L) return(NA_integer_)
      idx[[1L]]
    }

    col_companies   <- pick_col(CREATIVE_HEADER_PATTERNS$companies)
    col_claims      <- pick_col(CREATIVE_HEADER_PATTERNS$claims)
    col_productions <- pick_col(CREATIVE_HEADER_PATTERNS$productions)
    col_relief      <- pick_col(CREATIVE_HEADER_PATTERNS$relief_gbp_m)
    col_status      <- pick_col(CREATIVE_HEADER_PATTERNS$status)

    take_num <- function(idx) {
      if (is.na(idx) || idx > ncol(data_rows)) return(rep(NA_real_, nrow(data_rows)))
      parse_num(data_rows[[idx]])
    }
    take_chr <- function(idx) {
      if (is.na(idx) || idx > ncol(data_rows)) return(rep(NA_character_, nrow(data_rows)))
      trimws(as.character(data_rows[[idx]]))
    }

    results[[length(results) + 1L]] <- data.frame(
      sector       = sec,
      tax_year     = yr_raw,
      companies    = take_num(col_companies),
      claims       = take_num(col_claims),
      productions  = take_num(col_productions),
      relief_gbp_m = take_num(col_relief),
      status       = take_chr(col_status),
      stringsAsFactors = FALSE
    )
  }

  if (length(results) == 0L) {
    cli::cli_abort("No Creative Industries sector tables parsed. Please file an issue.")
  }

  out <- do.call(rbind, results)
  out <- out[order(out$sector, out$tax_year), ]
  rownames(out) <- NULL
  out
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hmrc_tobacco_duties.R
\name{hmrc_tobacco_duties}
\alias{hmrc_tobacco_duties}
\title{Download HMRC tobacco duty receipts}
\source{
\url{https://www.gov.uk/government/statistics/tobacco-bulletin}
}
\usage{
hmrc_tobacco_duties(product = NULL, start = NULL, end = NULL, cache = TRUE)
}
\arguments{
\item{product}{Character vector or \code{NULL} (default = all products).
Valid values: \code{"cigarettes"}, \code{"cigars"}, \code{"hand_rolling"},
\code{"other"}, \code{"total"}.}

\item{start}{Character \code{"YYYY-MM"} or a \code{Date} object.}

\item{end}{Character \code{"YYYY-MM"} or a \code{Date} object.}

\item{cache}{Logical. Use cached file if available (default \code{TRUE}).}
}
\value{
An \code{hmrc_tbl} with columns \code{date}, \code{product}, \code{description},
\code{receipts_gbp_m}.
}
\description{
Downloads and tidies the HMRC Tobacco Bulletin, which reports monthly
tobacco products duty receipts by product type. Data runs from January
1991 to the most recent published month, updated twice per year
(February and August).
}
\examples{
\donttest{
op <- options(hmrc.cache_dir = tempdir())
hmrc_tobacco_duties(start = "2015-01")
hmrc_tobacco_duties(product = "cigarettes")
options(op)
}

}
\seealso{
Other data fetchers: 
\code{\link{hmrc_capital_gains}()},
\code{\link{hmrc_corporation_tax}()},
\code{\link{hmrc_creative_industries}()},
\code{\link{hmrc_fuel_duties}()},
\code{\link{hmrc_income_tax_stats}()},
\code{\link{hmrc_inheritance_tax}()},
\code{\link{hmrc_patent_box}()},
\code{\link{hmrc_property_transactions}()},
\code{\link{hmrc_rd_credits}()},
\code{\link{hmrc_stamp_duty}()},
\code{\link{hmrc_tax_gap}()},
\code{\link{hmrc_tax_receipts}()},
\code{\link{hmrc_vat}()}
}
\concept{data fetchers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hmrc_income_tax_stats.R
\name{hmrc_income_tax_stats}
\alias{hmrc_income_tax_stats}
\title{Download HMRC Income Tax liabilities by income range}
\source{
\url{https://www.gov.uk/government/statistics/income-tax-liabilities-by-income-range}
}
\usage{
hmrc_income_tax_stats(tax_year = NULL, cache = TRUE)
}
\arguments{
\item{tax_year}{Character vector or \code{NULL} (default = all years). Filter to
specific tax years, e.g. \code{"2023-24"}.}

\item{cache}{Logical. Use cached file if available (default \code{TRUE}).}
}
\value{
An \code{hmrc_tbl} with columns \code{tax_year}, \code{income_range},
\code{income_lower_gbp}, \code{taxpayers_thousands}, \code{total_income_gbp_m},
\code{tax_liability_gbp_m}, \code{average_rate_pct}, \code{average_tax_gbp}.
}
\description{
Downloads and tidies HMRC Table 2.5, which reports the number of Income Tax
payers and their liabilities grouped by total income range, for each
available tax year. Numbers of taxpayers are in thousands; amounts are in
millions of pounds unless otherwise noted. Published annually in May/June.
}
\details{
The earliest tax year with outturn data is based on the Survey of Personal
Incomes; later years are projected estimates. Values suppressed for small
sample sizes are returned as \code{NA}.
}
\examples{
\donttest{
op <- options(hmrc.cache_dir = tempdir())
hmrc_income_tax_stats()
hmrc_income_tax_stats(tax_year = "2023-24")
options(op)
}

}
\seealso{
Other data fetchers: 
\code{\link{hmrc_capital_gains}()},
\code{\link{hmrc_corporation_tax}()},
\code{\link{hmrc_creative_industries}()},
\code{\link{hmrc_fuel_duties}()},
\code{\link{hmrc_inheritance_tax}()},
\code{\link{hmrc_patent_box}()},
\code{\link{hmrc_property_transactions}()},
\code{\link{hmrc_rd_credits}()},
\code{\link{hmrc_stamp_duty}()},
\code{\link{hmrc_tax_gap}()},
\code{\link{hmrc_tax_receipts}()},
\code{\link{hmrc_tobacco_duties}()},
\code{\link{hmrc_vat}()}
}
\concept{data fetchers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/catalogue.R
\docType{data}
\name{catalogue}
\alias{catalogue}
\title{HMRC dataset catalogue}
\format{
A data frame with columns:
\describe{
\item{dataset}{Character. Short identifier.}
\item{publication}{Character. GOV.UK publication name.}
\item{function_name}{Character. Exporting function (or \code{NA}).}
\item{frequency}{Character. \code{monthly}, \code{quarterly}, \code{annual}, etc.}
\item{start}{Character. First period of the published series.}
\item{publisher}{Character. Usually \code{"HMRC"}.}
\item{slug}{Character. GOV.UK statistics slug.}
\item{url}{Character. URL of the publication landing page.}
\item{description}{Character. One-line description.}
\item{tags}{Character. Space-separated keywords searched by
\code{\link[=hmrc_search]{hmrc_search()}}.}
}
}
\usage{
catalogue
}
\description{
A data frame describing every HMRC dataset known to the package, including
those exposed by an \verb{hmrc_*} function and those on the development roadmap
(where \code{function_name} is \code{NA}). Use \code{\link[=hmrc_search]{hmrc_search()}} for keyword search and
\code{\link[=hmrc_publications]{hmrc_publications()}} for a tidy index.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hmrc_inheritance_tax.R
\name{hmrc_inheritance_tax}
\alias{hmrc_inheritance_tax}
\title{Download HMRC Inheritance Tax liabilities by estate band}
\source{
\url{https://www.gov.uk/government/statistics/inheritance-tax-liabilities-statistics}
}
\usage{
hmrc_inheritance_tax(cache = TRUE)
}
\arguments{
\item{cache}{Logical. Use cached file if available (default \code{TRUE}).}
}
\value{
An \code{hmrc_tbl} (long format) with columns:
\describe{
\item{tax_year}{Character. Year of death, e.g. \code{"2022-23"}.}
\item{estate_band_lower_gbp}{Numeric. Lower limit of the net-estate
band in pounds.}
\item{estate_band}{Character. Plain-English label, e.g.
\code{"GBP 0-100k"}, \code{"Total"}.}
\item{measure}{Character. One of \code{"number_not_taxed"},
\code{"number_taxed"}, \code{"tax_due_gbp_m"}, \code{"avg_tax_gbp"},
\code{"effective_rate_pct"}.}
\item{value}{Numeric. Value, with \verb{[z]} (no tax due to NRB) and
\verb{[c]} (disclosure suppression) returned as \code{NA}.}
}
}
\description{
Downloads and tidies HMRC Table 12.1a of the Inheritance Tax Liabilities
Statistics: numbers of estates, tax due, average tax, and average
effective tax rate, broken down by net-estate band, for the latest
published year of death. Published annually in July, ~3 years after the
year of death due to the administration window.
}
\examples{
\donttest{
op <- options(hmrc.cache_dir = tempdir())
hmrc_inheritance_tax()
options(op)
}

}
\seealso{
Other data fetchers: 
\code{\link{hmrc_capital_gains}()},
\code{\link{hmrc_corporation_tax}()},
\code{\link{hmrc_creative_industries}()},
\code{\link{hmrc_fuel_duties}()},
\code{\link{hmrc_income_tax_stats}()},
\code{\link{hmrc_patent_box}()},
\code{\link{hmrc_property_transactions}()},
\code{\link{hmrc_rd_credits}()},
\code{\link{hmrc_stamp_duty}()},
\code{\link{hmrc_tax_gap}()},
\code{\link{hmrc_tax_receipts}()},
\code{\link{hmrc_tobacco_duties}()},
\code{\link{hmrc_vat}()}
}
\concept{data fetchers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hmrc_corporation_tax.R
\name{hmrc_corporation_tax}
\alias{hmrc_corporation_tax}
\title{Download HMRC Corporation Tax receipts by type}
\source{
\url{https://www.gov.uk/government/collections/analyses-of-corporation-tax-receipts-and-liabilities}
}
\usage{
hmrc_corporation_tax(cache = TRUE)
}
\arguments{
\item{cache}{Logical. Use cached file if available (default \code{TRUE}).}
}
\value{
An \code{hmrc_tbl} with columns \code{tax_year}, \code{type}, \code{description},
\code{receipts_gbp_m}.
}
\description{
Downloads and tidies the HMRC Corporation Tax Statistics annual
publication, reporting receipts broken down by tax type for the
most recent six financial years. Published annually in September.
}
\details{
Some levies (e.g. Residential Property Developer Tax, Electricity
Generators Levy) were introduced mid-series and have \code{NA} values for
earlier years.
}
\examples{
\donttest{
op <- options(hmrc.cache_dir = tempdir())
hmrc_corporation_tax()
options(op)
}

}
\seealso{
Other data fetchers: 
\code{\link{hmrc_capital_gains}()},
\code{\link{hmrc_creative_industries}()},
\code{\link{hmrc_fuel_duties}()},
\code{\link{hmrc_income_tax_stats}()},
\code{\link{hmrc_inheritance_tax}()},
\code{\link{hmrc_patent_box}()},
\code{\link{hmrc_property_transactions}()},
\code{\link{hmrc_rd_credits}()},
\code{\link{hmrc_stamp_duty}()},
\code{\link{hmrc_tax_gap}()},
\code{\link{hmrc_tax_receipts}()},
\code{\link{hmrc_tobacco_duties}()},
\code{\link{hmrc_vat}()}
}
\concept{data fetchers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hmrc-package.R
\docType{package}
\name{hmrc-package}
\alias{hmrc}
\alias{hmrc-package}
\title{hmrc: Download and Tidy HMRC Statistical Data}
\description{
Provides functions to download, parse, and tidy statistical data published
by HM Revenue and Customs (HMRC) on GOV.UK. Returns annotated \code{hmrc_tbl}
data frames with provenance metadata (source URL, fetch time, vintage,
cell methods) for reproducible fiscal research. File URLs are resolved at
runtime via the GOV.UK Content API, so data is always current.
}
\section{Data fetchers}{

\itemize{
\item \code{\link[=hmrc_tax_receipts]{hmrc_tax_receipts()}} — monthly tax receipts and NICs (April 2008+)
\item \code{\link[=hmrc_property_transactions]{hmrc_property_transactions()}} — monthly property transactions (April 2005+)
\item \code{\link[=hmrc_income_tax_stats]{hmrc_income_tax_stats()}} — annual Income Tax liabilities by income range
\item \code{\link[=hmrc_tax_gap]{hmrc_tax_gap()}} — annual tax gap estimates by tax type
\item \code{\link[=hmrc_stamp_duty]{hmrc_stamp_duty()}} — annual stamp duty receipts (SDLT, SDRT)
\item \code{\link[=hmrc_vat]{hmrc_vat()}} — monthly VAT receipts breakdown (April 1973+)
\item \code{\link[=hmrc_rd_credits]{hmrc_rd_credits()}} — annual R&D tax credit statistics (2000-01+)
\item \code{\link[=hmrc_fuel_duties]{hmrc_fuel_duties()}} — monthly fuel duty receipts (January 1990+)
\item \code{\link[=hmrc_tobacco_duties]{hmrc_tobacco_duties()}} — monthly tobacco duty receipts (January 1991+)
\item \code{\link[=hmrc_corporation_tax]{hmrc_corporation_tax()}} — annual Corporation Tax receipts by type
\item \code{\link[=hmrc_capital_gains]{hmrc_capital_gains()}} — annual CGT taxpayers, gains, liabilities (1987-88+)
\item \code{\link[=hmrc_inheritance_tax]{hmrc_inheritance_tax()}} — IHT estates and liabilities by net-estate band
\item \code{\link[=hmrc_patent_box]{hmrc_patent_box()}} — annual Patent Box election counts and relief
\item \code{\link[=hmrc_creative_industries]{hmrc_creative_industries()}} — annual Film/HETV/Games/Theatre/etc reliefs
}
}

\section{Discovery and infrastructure}{

\itemize{
\item \code{\link[=hmrc_search]{hmrc_search()}} — keyword search of the dataset catalogue
\item \code{\link[=hmrc_publications]{hmrc_publications()}} — index of implemented + planned publications
\item \code{\link[=hmrc_list_tax_heads]{hmrc_list_tax_heads()}} — lookup table of tax-receipts identifiers
\item \code{\link[=hmrc_cache_info]{hmrc_cache_info()}} — inspect locally cached files
\item \code{\link[=hmrc_clear_cache]{hmrc_clear_cache()}} — manage locally cached files
\item \code{\link[=hmrc_meta]{hmrc_meta()}} — extract provenance metadata from any \code{hmrc_tbl} result
}
}

\section{Citation}{

Run \code{citation("hmrc")} for the structured citation, or see the
\code{CITATION.cff} file at the package root for the GitHub citation widget.
}

\section{Data source}{

All data is published by HMRC on GOV.UK under the Open Government Licence.
See \url{https://www.gov.uk/government/organisations/hm-revenue-customs/about/statistics}.
}

\seealso{
Useful links:
\itemize{
  \item \url{https://charlescoverdale.github.io/hmrc/}
  \item \url{https://github.com/charlescoverdale/hmrc}
  \item Report bugs at \url{https://github.com/charlescoverdale/hmrc/issues}
}

}
\author{
\strong{Maintainer}: Charles Coverdale \email{charlesfcoverdale@gmail.com}

}
\concept{HMRC}
\concept{UK tax}
\concept{VAT}
\concept{corporation tax}
\concept{fuel duty}
\concept{government revenue}
\concept{income tax}
\concept{stamp duty}
\concept{tax revenue}
\concept{tobacco duty}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hmrc_property_transactions.R
\name{hmrc_property_transactions}
\alias{hmrc_property_transactions}
\title{Download monthly UK property transaction counts}
\source{
\url{https://www.gov.uk/government/statistics/monthly-property-transactions-completed-in-the-uk-with-value-40000-or-above}
}
\usage{
hmrc_property_transactions(
  type = c("all", "residential", "non_residential"),
  nation = NULL,
  start = NULL,
  end = NULL,
  cache = TRUE
)
}
\arguments{
\item{type}{Character. One of \code{"all"} (default), \code{"residential"}, or
\code{"non_residential"}.}

\item{nation}{Character vector or \code{NULL} (default = all nations).
Valid values: \code{"uk"}, \code{"england"}, \code{"scotland"}, \code{"wales"},
\code{"northern_ireland"}.}

\item{start}{Character \code{"YYYY-MM"} or a \code{Date} object.}

\item{end}{Character \code{"YYYY-MM"} or a \code{Date} object.}

\item{cache}{Logical. Use cached file if available (default \code{TRUE}).}
}
\value{
An \code{hmrc_tbl} with columns \code{date}, \code{nation}, \code{type},
\code{transactions}.
}
\description{
Downloads and tidies the HMRC Monthly Property Transactions bulletin,
which counts residential and non-residential property transactions
(SDLT returns, LBTT in Scotland, LTT in Wales) for England, Scotland,
Wales, Northern Ireland, and the UK total. Data runs from April 2005
to the most recent completed month.
}
\examples{
\donttest{
op <- options(hmrc.cache_dir = tempdir())
hmrc_property_transactions()
hmrc_property_transactions(type = "residential", nation = "england",
                           start = "2020-01")
options(op)
}

}
\seealso{
Other data fetchers: 
\code{\link{hmrc_capital_gains}()},
\code{\link{hmrc_corporation_tax}()},
\code{\link{hmrc_creative_industries}()},
\code{\link{hmrc_fuel_duties}()},
\code{\link{hmrc_income_tax_stats}()},
\code{\link{hmrc_inheritance_tax}()},
\code{\link{hmrc_patent_box}()},
\code{\link{hmrc_rd_credits}()},
\code{\link{hmrc_stamp_duty}()},
\code{\link{hmrc_tax_gap}()},
\code{\link{hmrc_tax_receipts}()},
\code{\link{hmrc_tobacco_duties}()},
\code{\link{hmrc_vat}()}
}
\concept{data fetchers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hmrc_vat.R
\name{hmrc_vat}
\alias{hmrc_vat}
\title{Download HMRC VAT receipts}
\source{
\url{https://www.gov.uk/government/statistics/value-added-tax-vat-annual-statistics}
}
\usage{
hmrc_vat(measure = NULL, start = NULL, end = NULL, cache = TRUE)
}
\arguments{
\item{measure}{Character vector or \code{NULL} (default = all measures).
Valid values: \code{"total"}, \code{"payments"}, \code{"repayments"},
\code{"import_vat"}, \code{"home_vat"}.}

\item{start}{Character \code{"YYYY-MM"} or a \code{Date} object.}

\item{end}{Character \code{"YYYY-MM"} or a \code{Date} object.}

\item{cache}{Logical. Use cached file if available (default \code{TRUE}).}
}
\value{
An \code{hmrc_tbl} with columns \code{date}, \code{measure}, \code{description},
\code{receipts_gbp_m}. Repayments are negative.
}
\description{
Downloads and tidies the HMRC VAT Annual Statistics bulletin, which
reports monthly VAT receipts broken down into payments, repayments,
import VAT, and home VAT. Monthly data runs from April 1973; the
bulletin is published annually (December).
}
\details{
Note that early years (pre-1985) have suppressed payment and repayment
splits; only the total is available for those periods. From January 2021,
import VAT collected via postponed VAT accounting is recorded within
payments and repayments rather than the import VAT column.
}
\examples{
\donttest{
op <- options(hmrc.cache_dir = tempdir())
hmrc_vat(measure = "total", start = "2010-01")
hmrc_vat(start = "2020-01")
options(op)
}

}
\seealso{
Other data fetchers: 
\code{\link{hmrc_capital_gains}()},
\code{\link{hmrc_corporation_tax}()},
\code{\link{hmrc_creative_industries}()},
\code{\link{hmrc_fuel_duties}()},
\code{\link{hmrc_income_tax_stats}()},
\code{\link{hmrc_inheritance_tax}()},
\code{\link{hmrc_patent_box}()},
\code{\link{hmrc_property_transactions}()},
\code{\link{hmrc_rd_credits}()},
\code{\link{hmrc_stamp_duty}()},
\code{\link{hmrc_tax_gap}()},
\code{\link{hmrc_tax_receipts}()},
\code{\link{hmrc_tobacco_duties}()}
}
\concept{data fetchers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hmrc_patent_box.R
\name{hmrc_patent_box}
\alias{hmrc_patent_box}
\title{Download HMRC Patent Box relief statistics}
\source{
\url{https://www.gov.uk/government/statistics/patent-box-reliefs-statistics}
}
\usage{
hmrc_patent_box(tax_year = NULL, cache = TRUE)
}
\arguments{
\item{tax_year}{Character vector or \code{NULL} (default = all years).
Filter to specific tax years, e.g. \code{"2023-24"}.}

\item{cache}{Logical. Use cached file if available (default \code{TRUE}).}
}
\value{
An \code{hmrc_tbl} with columns:
\describe{
\item{tax_year}{Character. Tax year, e.g. \code{"2023-24"}.}
\item{companies}{Numeric. Number of companies electing into the regime
(rounded to nearest 5 by HMRC).}
\item{relief_gbp_m}{Numeric. Total relief in millions of pounds.}
}
}
\description{
Downloads and tidies Table 1 of the HMRC Patent Box Relief Statistics:
annual number of companies electing into the Patent Box regime and the
total relief claimed (in GBP million). The Patent Box was introduced
from 1 April 2013; data runs from tax year 2013-14 to the most recent
published year (typically with a one-year provisional lag). Published
annually each September.
}
\examples{
\donttest{
op <- options(hmrc.cache_dir = tempdir())
hmrc_patent_box()
hmrc_patent_box(tax_year = "2022-23")
options(op)
}

}
\seealso{
Other data fetchers: 
\code{\link{hmrc_capital_gains}()},
\code{\link{hmrc_corporation_tax}()},
\code{\link{hmrc_creative_industries}()},
\code{\link{hmrc_fuel_duties}()},
\code{\link{hmrc_income_tax_stats}()},
\code{\link{hmrc_inheritance_tax}()},
\code{\link{hmrc_property_transactions}()},
\code{\link{hmrc_rd_credits}()},
\code{\link{hmrc_stamp_duty}()},
\code{\link{hmrc_tax_gap}()},
\code{\link{hmrc_tax_receipts}()},
\code{\link{hmrc_tobacco_duties}()},
\code{\link{hmrc_vat}()}
}
\concept{data fetchers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hmrc_tax_gap.R
\name{hmrc_tax_gap}
\alias{hmrc_tax_gap}
\title{Download HMRC tax gap estimates}
\source{
\url{https://www.gov.uk/government/statistics/measuring-tax-gaps}
}
\usage{
hmrc_tax_gap(tax = NULL, cache = TRUE)
}
\arguments{
\item{tax}{Character vector or \code{NULL} (default = all taxes). Filter by
tax type, e.g. \code{"Income Tax"}, \code{"VAT"}, \code{"Corporation Tax"}. Use
\code{unique(hmrc_tax_gap()$tax)} to see all available values.}

\item{cache}{Logical. Use cached file if available (default \code{TRUE}).}
}
\value{
An \code{hmrc_tbl} with columns \code{tax_year}, \code{tax}, \code{taxpayer_type},
\code{component}, \code{gap_pct}, \code{gap_gbp_bn}, \code{uncertainty}.
}
\description{
Downloads and tidies the HMRC Measuring the Tax Gap publication, which
estimates the difference between the tax theoretically owed and the tax
actually collected, broken down by tax type and taxpayer group. Published
annually in June, covering the most recent financial year.
}
\details{
The tax gap publication is cross-sectional: each edition covers a single
financial year. This function returns data for the most recent edition
available on GOV.UK. Historical estimates back to 2005-06 are available
in a separate HMRC publication (Measuring the Tax Gap: Time Series).
}
\examples{
\donttest{
op <- options(hmrc.cache_dir = tempdir())
hmrc_tax_gap()
hmrc_tax_gap(tax = "VAT")
options(op)
}

}
\seealso{
Other data fetchers: 
\code{\link{hmrc_capital_gains}()},
\code{\link{hmrc_corporation_tax}()},
\code{\link{hmrc_creative_industries}()},
\code{\link{hmrc_fuel_duties}()},
\code{\link{hmrc_income_tax_stats}()},
\code{\link{hmrc_inheritance_tax}()},
\code{\link{hmrc_patent_box}()},
\code{\link{hmrc_property_transactions}()},
\code{\link{hmrc_rd_credits}()},
\code{\link{hmrc_stamp_duty}()},
\code{\link{hmrc_tax_receipts}()},
\code{\link{hmrc_tobacco_duties}()},
\code{\link{hmrc_vat}()}
}
\concept{data fetchers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hmrc_creative_industries.R
\name{hmrc_creative_industries}
\alias{hmrc_creative_industries}
\title{Download HMRC Creative Industries tax relief statistics}
\source{
\url{https://www.gov.uk/government/collections/creative-industries-statistics}
}
\usage{
hmrc_creative_industries(sector = NULL, tax_year = NULL, cache = TRUE)
}
\arguments{
\item{sector}{Character vector or \code{NULL} (default = all sectors).
Valid values: \code{"film"}, \code{"high_end_tv"}, \code{"animation"},
\code{"childrens_tv"}, \code{"video_games"}, \code{"theatre"}, \code{"orchestra"},
\code{"museum"}.}

\item{tax_year}{Character vector or \code{NULL} (default = all years).}

\item{cache}{Logical. Use cached file if available (default \code{TRUE}).}
}
\value{
An \code{hmrc_tbl} with columns:
\describe{
\item{sector}{Character. Creative-industries sector identifier.}
\item{tax_year}{Character. Tax year, e.g. \code{"2023-24"}.}
\item{companies}{Numeric. Number of companies (where reported).}
\item{claims}{Numeric. Number of claims (where reported).}
\item{productions}{Numeric. Number of productions (films, games,
etc., where reported).}
\item{relief_gbp_m}{Numeric. Amount of relief paid, GBP million.}
\item{status}{Character. HMRC revision status (e.g. \code{"Unchanged"},
\code{"Provisional"}).}
}
}
\description{
Downloads and tidies Table 1 (annual time series) of the HMRC Creative
Industries Statistics for each of the eight creative-industries reliefs:
Film, High-end TV, Animation, Children's TV, Video Games, Theatre,
Orchestra, and Museums and Galleries Exhibition. Published annually
each August.
}
\details{
Creative Industries reliefs are paid on an accruals basis. The latest
tax year in each sector's table is provisional and uplifted by HMRC for
claims not yet received; status is recorded in the \code{status} column.
Sector tables differ slightly in their column set (films track
\code{productions}, video games track \code{productions}, theatre tracks
\code{claims}/\code{productions}, museums track exhibitions, etc.); columns absent
from a sector's table are returned as \code{NA}.
}
\examples{
\donttest{
op <- options(hmrc.cache_dir = tempdir())
hmrc_creative_industries(sector = "film")
hmrc_creative_industries(tax_year = "2023-24")
options(op)
}

}
\seealso{
Other data fetchers: 
\code{\link{hmrc_capital_gains}()},
\code{\link{hmrc_corporation_tax}()},
\code{\link{hmrc_fuel_duties}()},
\code{\link{hmrc_income_tax_stats}()},
\code{\link{hmrc_inheritance_tax}()},
\code{\link{hmrc_patent_box}()},
\code{\link{hmrc_property_transactions}()},
\code{\link{hmrc_rd_credits}()},
\code{\link{hmrc_stamp_duty}()},
\code{\link{hmrc_tax_gap}()},
\code{\link{hmrc_tax_receipts}()},
\code{\link{hmrc_tobacco_duties}()},
\code{\link{hmrc_vat}()}
}
\concept{data fetchers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hmrc_tax_receipts.R
\name{hmrc_tax_receipts}
\alias{hmrc_tax_receipts}
\title{Download HMRC tax receipts and National Insurance contributions}
\source{
\url{https://www.gov.uk/government/statistics/hmrc-tax-and-nics-receipts-for-the-uk}
}
\usage{
hmrc_tax_receipts(tax = NULL, start = NULL, end = NULL, cache = TRUE)
}
\arguments{
\item{tax}{Character vector of tax head identifiers, or \code{NULL} (default)
to return all available series. Use \code{\link[=hmrc_list_tax_heads]{hmrc_list_tax_heads()}} to see valid
values and descriptions.}

\item{start}{Character \code{"YYYY-MM"} or a \code{Date} object. If provided, rows
before this month are dropped.}

\item{end}{Character \code{"YYYY-MM"} or a \code{Date} object. If provided, rows
after this month are dropped.}

\item{cache}{Logical. If \code{TRUE} (default), the downloaded file is cached
locally and reused on subsequent calls. Use \code{\link[=hmrc_clear_cache]{hmrc_clear_cache()}} to
reset.}
}
\value{
An \code{hmrc_tbl} (subclass of \code{data.frame}) with columns:
\describe{
\item{date}{\code{Date}. The first day of the reference month.}
\item{tax_head}{Character. Tax or duty identifier (see
\code{\link[=hmrc_list_tax_heads]{hmrc_list_tax_heads()}}).}
\item{description}{Character. Plain-English series label.}
\item{receipts_gbp_m}{Numeric. Cash receipts in millions of pounds (GBP).}
}
Provenance metadata (source URL, fetch time, vintage) is attached as
the \code{"hmrc_meta"} attribute and can be inspected with \code{\link[=hmrc_meta]{hmrc_meta()}}.
}
\description{
Downloads and tidies the monthly HMRC Tax Receipts and National Insurance
Contributions bulletin published on GOV.UK. The bulletin covers all major
UK taxes and duties from April 2008 to the most recent month (monthly
granularity), updated on approximately the 15th working day of each month.
}
\examples{
\donttest{
op <- options(hmrc.cache_dir = tempdir())
hmrc_tax_receipts()
hmrc_tax_receipts(tax = c("income_tax", "vat"))
hmrc_tax_receipts(start = "2020-04")
hmrc_tax_receipts(tax = "vat", start = "2019-01", end = "2024-12")
options(op)
}

}
\seealso{
Other data fetchers: 
\code{\link{hmrc_capital_gains}()},
\code{\link{hmrc_corporation_tax}()},
\code{\link{hmrc_creative_industries}()},
\code{\link{hmrc_fuel_duties}()},
\code{\link{hmrc_income_tax_stats}()},
\code{\link{hmrc_inheritance_tax}()},
\code{\link{hmrc_patent_box}()},
\code{\link{hmrc_property_transactions}()},
\code{\link{hmrc_rd_credits}()},
\code{\link{hmrc_stamp_duty}()},
\code{\link{hmrc_tax_gap}()},
\code{\link{hmrc_tobacco_duties}()},
\code{\link{hmrc_vat}()}
}
\concept{data fetchers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hmrc_fuel_duties.R
\name{hmrc_fuel_duties}
\alias{hmrc_fuel_duties}
\title{Download HMRC hydrocarbon oil (fuel duty) receipts}
\source{
\url{https://www.gov.uk/government/statistics/hydrocarbon-oils-bulletin}
}
\usage{
hmrc_fuel_duties(fuel = NULL, start = NULL, end = NULL, cache = TRUE)
}
\arguments{
\item{fuel}{Character vector or \code{NULL} (default = all).
Valid values: \code{"total"}, \code{"petrol"}, \code{"diesel"}, \code{"other"}.}

\item{start}{Character \code{"YYYY-MM"} or a \code{Date} object.}

\item{end}{Character \code{"YYYY-MM"} or a \code{Date} object.}

\item{cache}{Logical. Use cached file if available (default \code{TRUE}).}
}
\value{
An \code{hmrc_tbl} with columns \code{date}, \code{fuel}, \code{description},
\code{receipts_gbp_m}.
}
\description{
Downloads and tidies the HMRC Hydrocarbon Oils Bulletin, which reports
monthly fuel duty receipts. Data runs from January 1990 to the most
recent published month, updated twice per year (January and July).
}
\examples{
\donttest{
op <- options(hmrc.cache_dir = tempdir())
hmrc_fuel_duties(fuel = "total", start = "2010-01")
hmrc_fuel_duties()
options(op)
}

}
\seealso{
Other data fetchers: 
\code{\link{hmrc_capital_gains}()},
\code{\link{hmrc_corporation_tax}()},
\code{\link{hmrc_creative_industries}()},
\code{\link{hmrc_income_tax_stats}()},
\code{\link{hmrc_inheritance_tax}()},
\code{\link{hmrc_patent_box}()},
\code{\link{hmrc_property_transactions}()},
\code{\link{hmrc_rd_credits}()},
\code{\link{hmrc_stamp_duty}()},
\code{\link{hmrc_tax_gap}()},
\code{\link{hmrc_tax_receipts}()},
\code{\link{hmrc_tobacco_duties}()},
\code{\link{hmrc_vat}()}
}
\concept{data fetchers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hmrc_cache_info.R
\name{hmrc_cache_info}
\alias{hmrc_cache_info}
\title{Inspect the local HMRC cache}
\usage{
hmrc_cache_info()
}
\value{
A data frame with columns \code{file}, \code{extension}, \code{size_bytes},
\code{size_mb}, \code{modified} (POSIXct), \code{age_days}, \code{path}. The data frame is
empty (zero rows) if the cache directory does not exist or is empty.
The cache directory itself is attached as the attribute \code{"cache_dir"}.
}
\description{
Returns a tidy table of files currently held in the package cache, with
size and age. Useful for understanding what is stored locally and deciding
when to clear stale files via \code{\link[=hmrc_clear_cache]{hmrc_clear_cache()}}.
}
\examples{
\donttest{
op <- options(hmrc.cache_dir = tempdir())
info <- hmrc_cache_info()
attr(info, "cache_dir")
options(op)
}

}
\seealso{
Other infrastructure: 
\code{\link{hmrc_clear_cache}()},
\code{\link{hmrc_list_tax_heads}()},
\code{\link{hmrc_meta}()},
\code{\link{hmrc_publications}()},
\code{\link{hmrc_search}()}
}
\concept{infrastructure}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hmrc_stamp_duty.R
\name{hmrc_stamp_duty}
\alias{hmrc_stamp_duty}
\title{Download HMRC stamp duty receipts}
\source{
\url{https://www.gov.uk/government/statistics/uk-stamp-tax-statistics}
}
\usage{
hmrc_stamp_duty(type = NULL, cache = TRUE)
}
\arguments{
\item{type}{Character vector or \code{NULL} (default = all types).
Valid values: \code{"sdlt_property"}, \code{"sdlt_leases"}, \code{"sdlt_total"},
\code{"sdrt"}, \code{"stamp_duty"}, \code{"total"}.}

\item{cache}{Logical. Use cached file if available (default \code{TRUE}).}
}
\value{
An \code{hmrc_tbl} with columns \code{tax_year}, \code{type}, \code{description},
\code{receipts_gbp_m} (rounded to nearest GBP 5m by HMRC).
}
\description{
Downloads and tidies the HMRC Annual Stamp Tax Statistics, covering
Stamp Duty Land Tax (SDLT), Stamp Duty Reserve Tax (SDRT) on shares,
and other stamp duties. Annual data from 2003-04 to the most recent
financial year, published each December.
}
\examples{
\donttest{
op <- options(hmrc.cache_dir = tempdir())
hmrc_stamp_duty()
hmrc_stamp_duty(type = "sdlt_total")
options(op)
}

}
\seealso{
Other data fetchers: 
\code{\link{hmrc_capital_gains}()},
\code{\link{hmrc_corporation_tax}()},
\code{\link{hmrc_creative_industries}()},
\code{\link{hmrc_fuel_duties}()},
\code{\link{hmrc_income_tax_stats}()},
\code{\link{hmrc_inheritance_tax}()},
\code{\link{hmrc_patent_box}()},
\code{\link{hmrc_property_transactions}()},
\code{\link{hmrc_rd_credits}()},
\code{\link{hmrc_tax_gap}()},
\code{\link{hmrc_tax_receipts}()},
\code{\link{hmrc_tobacco_duties}()},
\code{\link{hmrc_vat}()}
}
\concept{data fetchers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hmrc_capital_gains.R
\name{hmrc_capital_gains}
\alias{hmrc_capital_gains}
\title{Download HMRC Capital Gains Tax statistics}
\source{
\url{https://www.gov.uk/government/organisations/hm-revenue-customs/series/capital-gains-tax-statistics}
}
\usage{
hmrc_capital_gains(tax_year = NULL, measure = NULL, cache = TRUE)
}
\arguments{
\item{tax_year}{Character vector or \code{NULL} (default = all years).
Filter to specific tax years, e.g. \code{"2022-23"}.}

\item{measure}{Character vector or \code{NULL} (default = all measures).
Valid values: \code{"taxpayers_total_thousands"},
\code{"taxpayers_individuals_thousands"}, \code{"taxpayers_trusts_thousands"},
\code{"males_pct"}, \code{"females_pct"},
\code{"gains_individuals_gbp_m"}, \code{"gains_trusts_gbp_m"},
\code{"gains_total_gbp_m"}, \code{"tax_individuals_gbp_m"},
\code{"tax_trusts_gbp_m"}, \code{"tax_total_gbp_m"}.}

\item{cache}{Logical. Use cached file if available (default \code{TRUE}).}
}
\value{
An \code{hmrc_tbl} (long format) with columns \code{tax_year}, \code{measure},
\code{value}. Numbers of taxpayers are in thousands; amounts in millions
of pounds; percentages in percent. Suppressed cells return \code{NA}.
}
\description{
Downloads and tidies HMRC Table 1 of the Capital Gains Tax statistics
bulletin: estimated number of CGT taxpayers, amounts of gains, and tax
liabilities by year of disposal. Data runs from 1987-88 onwards and is
published annually each summer.
}
\details{
Gains are after the deduction of taper relief and losses plus attributed
gains, but before deduction of the Annual Exempt Amount. Trusts include
personal representatives of the deceased. Statistics for the latest two
tax years are provisional and are revised in subsequent publications as
late returns arrive.
}
\examples{
\donttest{
op <- options(hmrc.cache_dir = tempdir())
hmrc_capital_gains()
hmrc_capital_gains(measure = "tax_total_gbp_m")
hmrc_capital_gains(tax_year = "2022-23")
options(op)
}

}
\seealso{
Other data fetchers: 
\code{\link{hmrc_corporation_tax}()},
\code{\link{hmrc_creative_industries}()},
\code{\link{hmrc_fuel_duties}()},
\code{\link{hmrc_income_tax_stats}()},
\code{\link{hmrc_inheritance_tax}()},
\code{\link{hmrc_patent_box}()},
\code{\link{hmrc_property_transactions}()},
\code{\link{hmrc_rd_credits}()},
\code{\link{hmrc_stamp_duty}()},
\code{\link{hmrc_tax_gap}()},
\code{\link{hmrc_tax_receipts}()},
\code{\link{hmrc_tobacco_duties}()},
\code{\link{hmrc_vat}()}
}
\concept{data fetchers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{tax_heads}
\alias{tax_heads}
\title{Tax head lookup table}
\format{
A data frame with 41 rows and 4 columns:
\describe{
\item{tax_head}{Character. Identifier used in the \code{tax} argument of
\code{\link[=hmrc_tax_receipts]{hmrc_tax_receipts()}}.}
\item{description}{Character. Plain-English description.}
\item{category}{Character. Broad grouping: \code{"income"}, \code{"nics"},
\code{"consumption"}, \code{"property"}, \code{"environment"}, \code{"expenditure"},
\code{"other"}, or \code{"total"}.}
\item{available_from}{Character. Approximate start year of monthly data.}
}
}
\source{
Derived from the HMRC Tax Receipts and NICs bulletin.
\url{https://www.gov.uk/government/statistics/hmrc-tax-and-nics-receipts-for-the-uk}
}
\usage{
tax_heads
}
\description{
A data frame describing all tax and duty series available in
\code{\link[=hmrc_tax_receipts]{hmrc_tax_receipts()}}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hmrc_list_tax_heads.R
\name{hmrc_list_tax_heads}
\alias{hmrc_list_tax_heads}
\title{List available tax heads in the HMRC Tax Receipts bulletin}
\usage{
hmrc_list_tax_heads()
}
\value{
A data frame with columns:
\describe{
\item{tax_head}{Character. Identifier used in the \code{tax} argument of
\code{\link[=hmrc_tax_receipts]{hmrc_tax_receipts()}}.}
\item{description}{Character. Plain-English description.}
\item{category}{Character. Broad grouping.}
\item{available_from}{Character. Earliest year of monthly data.}
}
}
\description{
Returns a data frame describing all tax and duty heads available in
\code{\link[=hmrc_tax_receipts]{hmrc_tax_receipts()}}. No network connection is required: the data is
bundled with the package.
}
\examples{
hmrc_list_tax_heads()

}
\seealso{
Other infrastructure: 
\code{\link{hmrc_cache_info}()},
\code{\link{hmrc_clear_cache}()},
\code{\link{hmrc_meta}()},
\code{\link{hmrc_publications}()},
\code{\link{hmrc_search}()}
}
\concept{infrastructure}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/catalogue.R
\name{hmrc_search}
\alias{hmrc_search}
\title{Search the HMRC dataset catalogue}
\usage{
hmrc_search(query = NULL, implemented = NULL, frequency = NULL)
}
\arguments{
\item{query}{Character. Keyword or regex pattern. Case-insensitive.
Matches anywhere in \code{dataset}, \code{publication}, \code{description}, or \code{tags}.
If \code{NULL} or empty, the full catalogue is returned.}

\item{implemented}{Logical or \code{NULL}. If \code{TRUE}, only rows with a
working \code{function_name} are returned. If \code{FALSE}, only roadmap rows.
\code{NULL} (default) returns both.}

\item{frequency}{Optional character vector to filter by frequency
(e.g. \code{"monthly"}, \code{"annual"}).}
}
\value{
A data frame, sorted with implemented datasets first.
}
\description{
Fuzzy keyword search across publication name, description, tags, and
dataset identifier. Returns a tidy table of matching rows so users can
discover what the package exposes (and what is on the roadmap).
}
\examples{
# Everything mentioning VAT
hmrc_search("vat")

# Only annual publications already implemented
hmrc_search(implemented = TRUE, frequency = "annual")

# Roadmap items mentioning capital gains
hmrc_search("capital gains", implemented = FALSE)

}
\seealso{
Other infrastructure: 
\code{\link{hmrc_cache_info}()},
\code{\link{hmrc_clear_cache}()},
\code{\link{hmrc_list_tax_heads}()},
\code{\link{hmrc_meta}()},
\code{\link{hmrc_publications}()}
}
\concept{infrastructure}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/catalogue.R
\name{hmrc_publications}
\alias{hmrc_publications}
\title{Index of HMRC publications known to the package}
\usage{
hmrc_publications(status = c("all", "implemented", "planned"))
}
\arguments{
\item{status}{One of \code{"all"} (default), \code{"implemented"}, or \code{"planned"}.}
}
\value{
A data frame with one row per publication and a \code{status} column
(\code{"implemented"} or \code{"planned"}).
}
\description{
Tidy roster of HMRC publications, marking which are exposed via an
\verb{hmrc_*} function and which are on the development roadmap. Useful for
planning analyses, citing the package, and tracking coverage over time.
}
\examples{
# Everything in the catalogue
hmrc_publications()

# Only roadmap items
hmrc_publications("planned")

}
\seealso{
Other infrastructure: 
\code{\link{hmrc_cache_info}()},
\code{\link{hmrc_clear_cache}()},
\code{\link{hmrc_list_tax_heads}()},
\code{\link{hmrc_meta}()},
\code{\link{hmrc_search}()}
}
\concept{infrastructure}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hmrc-tbl.R
\name{hmrc_meta}
\alias{hmrc_meta}
\title{Provenance metadata of an \code{hmrc_tbl}}
\usage{
hmrc_meta(x)
}
\arguments{
\item{x}{An object returned by an \verb{hmrc_*} data function.}
}
\value{
A named list, or \code{NULL} if \code{x} has no metadata.
}
\description{
Returns the provenance list attached to a result from any \verb{hmrc_*} data
fetcher: source URL, fetch time, vintage, cell methods, and so on. Useful
for citation, audit trails, and reproducibility.
}
\examples{
\donttest{
op <- options(hmrc.cache_dir = tempdir())
tr <- hmrc_tax_receipts(tax = "vat", start = "2024-01")
hmrc_meta(tr)
options(op)
}

}
\seealso{
Other infrastructure: 
\code{\link{hmrc_cache_info}()},
\code{\link{hmrc_clear_cache}()},
\code{\link{hmrc_list_tax_heads}()},
\code{\link{hmrc_publications}()},
\code{\link{hmrc_search}()}
}
\concept{infrastructure}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprec.R
\name{deprecated}
\alias{deprecated}
\alias{get_tax_receipts}
\alias{get_vat}
\alias{get_fuel_duties}
\alias{get_tobacco_duties}
\alias{get_corporation_tax}
\alias{get_stamp_duty}
\alias{get_property_transactions}
\alias{get_rd_credits}
\alias{get_tax_gap}
\alias{get_income_tax_stats}
\alias{list_tax_heads}
\alias{clear_cache}
\title{Deprecated get_* aliases}
\usage{
get_tax_receipts(...)

get_vat(...)

get_fuel_duties(...)

get_tobacco_duties(...)

get_corporation_tax(...)

get_stamp_duty(...)

get_property_transactions(...)

get_rd_credits(...)

get_tax_gap(...)

get_income_tax_stats(...)

list_tax_heads()

clear_cache(max_age_days = NULL)
}
\description{
Renamed to use the \verb{hmrc_*} prefix in v0.4.0. These aliases will be
removed in v0.6.0.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hmrc_clear_cache.R
\name{hmrc_clear_cache}
\alias{hmrc_clear_cache}
\title{Clear the local HMRC data cache}
\usage{
hmrc_clear_cache(max_age_days = NULL)
}
\arguments{
\item{max_age_days}{Numeric or \code{NULL}. If \code{NULL} (default), every cached
file is removed. If a number, only files modified more than that many
days ago are removed.}
}
\value{
Invisibly returns the number of files deleted.
}
\description{
Deletes locally cached data files downloaded by the package. By default
all cached files are removed; pass \code{max_age_days} to remove only files
older than that.
}
\examples{
\donttest{
op <- options(hmrc.cache_dir = tempdir())
hmrc_clear_cache()
hmrc_clear_cache(max_age_days = 30)
options(op)
}

}
\seealso{
Other infrastructure: 
\code{\link{hmrc_cache_info}()},
\code{\link{hmrc_list_tax_heads}()},
\code{\link{hmrc_meta}()},
\code{\link{hmrc_publications}()},
\code{\link{hmrc_search}()}
}
\concept{infrastructure}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hmrc_rd_credits.R
\name{hmrc_rd_credits}
\alias{hmrc_rd_credits}
\title{Download HMRC R&D tax credit statistics}
\source{
\url{https://www.gov.uk/government/statistics/corporate-tax-research-and-development-tax-credit}
}
\usage{
hmrc_rd_credits(scheme = NULL, measure = NULL, cache = TRUE)
}
\arguments{
\item{scheme}{Character vector or \code{NULL} (default = all schemes).
Valid values: \code{"sme"}, \code{"rdec"}, \code{"total"}.}

\item{measure}{Character vector or \code{NULL} (default = all measures).
Valid values: \code{"claims"}, \code{"amount_gbp_m"}.}

\item{cache}{Logical. Use cached file if available (default \code{TRUE}).}
}
\value{
An \code{hmrc_tbl} with columns \code{tax_year}, \code{scheme}, \code{description},
\code{measure}, \code{value}.
}
\description{
Downloads and tidies the HMRC Research and Development Tax Credits
Statistics publication, covering the SME R&D Relief and Research and
Development Expenditure Credit (RDEC) schemes. Annual data runs from
2000-01 to the most recent financial year, published annually in September.
}
\details{
Data before 2003-04 covers only the SME scheme (RDEC was introduced in
2002). Figures for the most recent two years are provisional and subject
to revision as late claims are processed.
}
\examples{
\donttest{
op <- options(hmrc.cache_dir = tempdir())
hmrc_rd_credits()
hmrc_rd_credits(scheme = "sme", measure = "claims")
options(op)
}

}
\seealso{
Other data fetchers: 
\code{\link{hmrc_capital_gains}()},
\code{\link{hmrc_corporation_tax}()},
\code{\link{hmrc_creative_industries}()},
\code{\link{hmrc_fuel_duties}()},
\code{\link{hmrc_income_tax_stats}()},
\code{\link{hmrc_inheritance_tax}()},
\code{\link{hmrc_patent_box}()},
\code{\link{hmrc_property_transactions}()},
\code{\link{hmrc_stamp_duty}()},
\code{\link{hmrc_tax_gap}()},
\code{\link{hmrc_tax_receipts}()},
\code{\link{hmrc_tobacco_duties}()},
\code{\link{hmrc_vat}()}
}
\concept{data fetchers}
